﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Text;

namespace Owl.Asf
{
	/// <summary>
	/// ASF ファイルのタグ情報を編集します。
	/// </summary>
	public class AsfTagEditor : IDisposable
	{
		/// <summary>
		/// インスタンスを初期化します。
		/// </summary>
		/// <param name="filePath">ASF ファイルのパス情報。</param>
		/// <exception cref="ArgumentNullException">fileName が null 参照です。</exception>
		/// <exception cref="ArgumentException">fileName が空の文字列 ("") か、空白しか含んでいないか、無効な文字を 1 つ以上含んでいます。またはファイル以外のデバイスを参照しています ( NTFS 環境の "con:"、"com1:"、"lpt1:" など )。</exception>
		/// <exception cref="NotSupportedException">fileName がファイル以外のデバイスを参照 ( NTFS 以外の環境の "con:"、"com1:"、"lpt1:" など )、または ASF 形式のファイルではありません。</exception>
		/// <exception cref="FileNotFoundException">ファイルが見つかりません。</exception>
		/// <exception cref="SecurityException">呼び出し元に、必要なアクセス許可がありません。</exception>
		/// <exception cref="PathTooLongException">指定したパス、ファイル名、またはその両方がシステム定義の最大長を超えています。たとえば、Windows ベースのプラットフォームの場合、パスの長さは 248 文字未満、ファイル名の長さは 260 文字未満である必要があります。</exception>
		public AsfTagEditor( string filePath )
		{
			this._reader  = this.Load( filePath );
			this.FilePath = filePath;
		}

		/// <summary>
		/// リソースを破棄します。
		/// </summary>
		public void Dispose()
		{
			if( this._reader != null )
			{
				this._reader.Dispose();
				this._reader = null;
			}

			if( this._tags != null )
			{
				this._tags.Clear();
				this._tags = null;
			}

			GC.SuppressFinalize( this );
		}

		/// <summary>
		/// 編集内容を保存するための一時ファイルのパスを取得します。
		/// </summary>
		/// <returns>成功時は一時ファイルのパス。それ以外は null。</returns>
		private string CreateTempFilePath()
		{
			string tempPath = null;
			{
				// ファイル重複を考慮し、1 秒間 ( 5 x 200 ms ) の猶予を持たせる
				var dir = Path.GetDirectoryName( this.FilePath );
				for( var i = 0; i < 5; ++i )
				{
					var path = String.Format( "{0}\\{1}", dir, Path.GetRandomFileName() );
					if( !File.Exists( path ) )
					{
						tempPath = path;
						break;
					}

					System.Threading.Thread.Sleep( 200 );
				}
			}

			return tempPath;
		}

		/// <summary>
		/// AST のタグ情報をダンプします。
		/// </summary>
		/// <param name="filePath">ASF ファイルへのパス情報。</param>
		/// <param name="dumpFilePath">ダンプ先となるテキストファイルへのパス情報。</param>
		/// <exception cref="NotSupportedException">ASF ファイルではありません。</exception>
		public static void Dump( string filePath, string dumpFilePath )
		{
			using( var dumper = new TagDumper( filePath, dumpFilePath ) )
			{
				dumper.Dump();
			}
		}

		/// <summary>
		/// タグ情報を読み込みます。
		/// </summary>
		/// <param name="path">ASF ファイル。</param>
		/// <returns>タグ情報を読み取るためのストリーム。</returns>
		/// <exception cref="NotSupportedException">ASF ファイルではありません。</exception>
		private Stream Load( string path )
		{
			// 書き込みを終えたあと、再初期化のために呼び出されることを想定しておく
			var reader = new FileStream( path, FileMode.Open, FileAccess.Read, FileShare.Read );
			this._tags.Clear();
			this.IsEdited = false;
			reader.Seek( 0, SeekOrigin.Begin );

			// ASF ファイルであることをチェック
			{
				var info = new HeaderObjectInfo( reader );
				if( info.Guid != Guids.HeaderObject ) { throw new NotSupportedException( "ASF file format is not." ); }
			}

			// オブジェクトの総数取得と Reserved 1、2 のスキップ
			var count = reader.ReadInt32();
			reader.Seek( 2, SeekOrigin.Current );

			// オブジェクト読み取り
			for( var i = 0; i < count; ++i )
			{
				var top = reader.Position;
				var info = new HeaderObjectInfo( reader );

				if( info.Guid == Guids.ContentDescriptionObject )
				{
					this.LoadContentDescription( reader );
				}
				else if( info.Guid == Guids.ExtendedContentDescriptionObject )
				{
					this.LoadExtendedContentDescription( reader );
				}
				else if( info.Guid == Guids.FilePropertiesObject )
				{
					this.LoadFileProperties( reader );
				}

				// 次のオブジェクト始点へ移動
				reader.Seek( top + info.Size, SeekOrigin.Begin );
			}

			return reader;
		}

		/// <summary>
		/// Content Description Object の内容を読み取ります。
		/// </summary>
		/// <param name="reader">タグ情報を読み取るためのストリーム。</param>
		private void LoadContentDescription( Stream reader )
		{
			var lengths = new int[ 5 ] { reader.ReadInt16(), reader.ReadInt16(), reader.ReadInt16(), reader.ReadInt16(), reader.ReadInt16() };
			var names   = new string[ 5 ] { AsfDefinedTags.Title.Name, AsfDefinedTags.Author.Name, AsfDefinedTags.Copyright.Name, AsfDefinedTags.Description.Name, AsfDefinedTags.Rating.Name };

			for( var i = 0; i < 5; ++i )
			{
				this._tags.Add( names[ i ], new TagData( reader.Position, lengths[ i ], TagDataType.String ) );
				reader.Seek( lengths[ i ], SeekOrigin.Current );
			}
		}

		/// <summary>
		/// Extended Content Description Object の内容を読み取ります。
		/// </summary>
		/// <param name="reader">タグ情報を読み取るためのストリーム。</param>
		private void LoadExtendedContentDescription( Stream reader )
		{
			ushort count = reader.ReadUInt16();
			for( ushort i = 0; i < count; ++i )
			{
				var name   = reader.ReadStringUtf16( reader.ReadUInt16() );
				var type   = ( TagDataType )reader.ReadUInt16();
				var length = reader.ReadUInt16();

				// 非公開のタグは読み飛ばす
				var info = AsfDefinedTags.GetInfo( name );
				if( info == null )
				{
					reader.Seek( length, SeekOrigin.Current );
					continue;
				}

				// 可変長のデータは遅延、それ以外はここで読み込む
				if( info.Type == AsfTagDataType.Binary || info.Type == AsfTagDataType.String )
				{
					this._tags.Add( name, new TagData( reader.Position, length, type ) );
					reader.Seek( length, SeekOrigin.Current );
				}
				else
				{
					this._tags.Add( name, new TagData( this.Read( reader.Position, length, info.Type, type, reader ) ) );
				}
			}
		}

		/// <summary>
		/// File Properties Object の内容を読み取ります。
		/// </summary>
		/// <param name="reader">タグ情報を読み取るためのストリーム。</param>
		private void LoadFileProperties( Stream reader )
		{
			// File ID をスキップ
			reader.Seek( 16, SeekOrigin.Current );

			// File Size
			this._tags.Add( AsfDefinedTags.FileSize.Name, new TagData( reader.ReadUInt64() ) );

			// Creation Date ( 8 )、Data Packets Count ( 8 ) をスキップ
			reader.Seek( 16, SeekOrigin.Current );

			// Play Duration
			var duration = reader.ReadUInt64();

			// Send Duration をスキップ
			reader.Seek( 8, SeekOrigin.Current );

			// Play Duration から Preroll x 10000 を差し引いたものが演奏時間となる。
			// この値は複数値からの算出が必要なので、遅延読み込みせずにキャッシュする。
			//
			duration -= ( reader.ReadUInt64() * 10000 );
			this._tags.Add( AsfDefinedTags.Duration.Name, new TagData( duration ) );

			// ... 現時点で他のデータは不要
		}

		/// <summary>
		/// タグ情報を読み取ります。
		/// </summary>
		/// <param name="tag">タグ。</param>
		/// <returns>成功時はタグ情報。それ以外は null 参照。</returns>
		/// <exception cref="ArgumentNullException">tag に null 参照が指定されました。</exception>
		public object Read( AsfTagInfo tag )
		{
			if( tag == null ) { throw new ArgumentNullException( "tag" ); }

			TagData data;
			if( !this._tags.TryGetValue( tag.Name, out data ) ) { return null; }

			var value = data.Value as TagReadData;
			return ( value == null ? data.Value : this.Read( value.Position, value.Length, tag.Type, value.Type, this._reader ) );
		}

		/// <summary>
		/// タグ情報を読み取ります。
		/// </summary>
		/// <param name="type">タグのデータ型。</param>
		/// <param name="data">タグ情報。</param>
		/// <returns>成功時はタグ情報。それ以外は null 参照。</returns>
		private object Read( long position, int length, AsfTagDataType type, TagDataType innerType, Stream reader )
		{
			reader.Seek( position, SeekOrigin.Begin );

			// 外部公開する値とファイルのデータ型に生じる齟齬を解決する。
			// 数値文字列、またはその逆など、相互変換が可能なものであればサポートする。
			//
			switch( type )
			{
			case AsfTagDataType.Binary:
				if( innerType == TagDataType.Binary ) { return reader.ReadBytes( length ); }
				break;

			case AsfTagDataType.Bool:
				switch( innerType )
				{
				case TagDataType.Bool:   return reader.ReadBool( length );
				case TagDataType.UInt16: return ( reader.ReadUInt16() != 0 );
				case TagDataType.UInt32: return ( reader.ReadUInt32() != 0 );
				case TagDataType.UInt64: return ( reader.ReadUInt64() != 0 );
				}
				break;

			case AsfTagDataType.Guid:
				switch( innerType )
				{
				case TagDataType.Binary: return reader.ReadGuid();
				case TagDataType.String: return new Guid( reader.ReadStringUtf16( length ) );
				}
				break;

			case AsfTagDataType.String:
				switch( innerType )
				{
				case TagDataType.Bool:   return reader.ReadBool().ToString();
				case TagDataType.String: return reader.ReadStringUtf16( length );
				case TagDataType.UInt16: return reader.ReadUInt16().ToString();
				case TagDataType.UInt32: return reader.ReadUInt32().ToString();
				case TagDataType.UInt64: return reader.ReadUInt64().ToString();
				}
				break;

			case AsfTagDataType.UInt16:
				switch( innerType )
				{
				case TagDataType.String:
					{
						ushort result;
						if( ushort.TryParse( reader.ReadStringUtf16( length ), out result ) ) { return result; }
					}
					break;

				case TagDataType.UInt16: return reader.ReadUInt16();
				case TagDataType.UInt32: return ( ushort )reader.ReadUInt32();
				case TagDataType.UInt64: return ( ushort )reader.ReadUInt64();
				}
				break;

			case AsfTagDataType.UInt32:
				switch( innerType )
				{
				case TagDataType.String:
					{
						uint result;
						if( uint.TryParse( reader.ReadStringUtf16( length ), out result ) ) { return result; }
					}
					break;

				case TagDataType.UInt16: return ( uint )reader.ReadUInt16();
				case TagDataType.UInt32: return reader.ReadUInt32();
				case TagDataType.UInt64: return ( uint )reader.ReadUInt64();
				}
				break;

			case AsfTagDataType.UInt64:
				switch( innerType )
				{
				case TagDataType.String:
					{
						ulong result;
						if( ulong.TryParse( reader.ReadStringUtf16( length ), out result ) )
						{
							return result;
						}
					}
					break;

				case TagDataType.UInt16: return ( ulong )reader.ReadUInt16();
				case TagDataType.UInt32: return ( ulong )reader.ReadUInt32();
				case TagDataType.UInt64: return reader.ReadUInt64();
				}
				break;
			}

			return null;
		}

		/// <summary>
		/// 編集内容を保存します
		/// </summary>
		/// <exception cref="IOException">編集内容を保存するための一時ファイル生成に失敗しました。</exception>
		public void Save()
		{
			// 未編集なら何もしない
			if( !this.IsEdited ) { return; }

			var tempFilePath = this.CreateTempFilePath();
			if( tempFilePath == null ) { throw new IOException( "Not create temp file." ); }

			// 一時ファイルに対して保存をおこなう
			try
			{
				using( var writer = new FileStream( tempFilePath, FileMode.Create, FileAccess.Write, FileShare.None ) )
				{
					var fileSizePosition = this.SaveHeader( this._reader, writer );
					this.SaveHeaderAfter( this._reader, writer );

					// ファイルサイズを更新
					writer.Seek( fileSizePosition, SeekOrigin.Begin );
					writer.Write( BitConverter.GetBytes( writer.Length ), 0, 8 );
				}
			}
			catch( Exception e )
			{
				File.Delete( tempFilePath );
				throw e;
			}

			// 一時ファイルを正式なファイルとして確定させる
			{
				var originalFilePath = this.CreateTempFilePath();
				if( originalFilePath == null )
				{
					File.Delete( tempFilePath );
					throw new IOException( "Could not create temporary files in order to escape the original file." );
				}

				// 作成日時を引き継ぐために記録
				DateTime creationTime;
				{
					var info = new FileInfo( this.FilePath );
					creationTime = info.CreationTime;
				}

				this._reader.Dispose();
				File.Move( this.FilePath, originalFilePath );
				File.Move( tempFilePath,  this.FilePath    );
				File.Delete( originalFilePath );

				// 作成日時の引き継ぎ
				{
					var info = new FileInfo( this.FilePath );
					info.CreationTime = creationTime;
				}

				// タグ情報とストリームを初期化
				this._reader = this.Load( this.FilePath );
			}		
		}

		/// <summary>
		/// Content Description Object の内容を保存します。
		/// </summary>
		/// <param name="reader">読み取りストリーム。</param>
		/// <param name="writer">書き込みストリーム。</param>
		/// <returns>Content Description Object のサイズ。</returns>
		private long SaveContentDescription( Stream reader, Stream writer, long originalSize )
		{
			var sizePos = reader.Position - 8;		// Object サイズの書き込み位置
			var values  = new List< byte[] >( 5 );	// 書き込むバイト配列リスト
			{
				var lengths  = new int[ 5 ]    { reader.ReadInt16(), reader.ReadInt16(), reader.ReadInt16(), reader.ReadInt16(), reader.ReadInt16() };
				var names    = new string[ 5 ] { AsfDefinedTags.Title.Name, AsfDefinedTags.Author.Name, AsfDefinedTags.Copyright.Name, AsfDefinedTags.Description.Name, AsfDefinedTags.Rating.Name };
				var emptyStr = new byte[ 0 ];

				for( var i = 0; i < names.Length; ++i )
				{
					TagData data;
					if( this._tags.TryGetValue( names[ i ], out data ) )
					{
						if( data.IsEdited )
						{
							// 値が編集済みなら、必ず文字列が入る
							values.Add( TagWriteData.ConvertStringUtf16ToBytes( ( string )data.Value ) );
							reader.Seek( lengths[ i ], SeekOrigin.Current );
						}
						else
						{
							values.Add( reader.ReadBytes( lengths[ i ] ) );
						}

						// Extended Content Description などで書き込み対象にならないように、値を削除
						this._tags.Remove( names[ i ] );
					}
					else
					{
						// このデータは常に領域を確保するので、値が削除されていた場合は空文字を書き込む
						values.Add( emptyStr );
						reader.Seek( lengths[ i ], SeekOrigin.Current );
					}

					// 文字数
					writer.Write( BitConverter.GetBytes( ( ushort )values[ i ].Length ), 0, 2 );
				}
			}

			// 文字列の書き込みと Object サイズ ( 初期値は「Header + 文字列長 ( 2 byte ) x 5」とする ) の算出
			long size = HeaderObjectInfo.ClassByteSize + 10;
			for( var i = 0; i < values.Count; ++i )
			{
				writer.Write( values[ i ], 0, values[ i ].Length );
				size += values[ i ].Length;
			}

			// Object サイズ更新
			{
				var pos = writer.Position;
				writer.Seek( sizePos, SeekOrigin.Begin );
				var b = BitConverter.GetBytes( size );
				writer.Write( b, 0, b.Length );
				writer.Seek( pos, SeekOrigin.Begin );
			}

			return size;
		}

		/// <summary>
		/// Extended Content Description Object の内容を保存します。
		/// </summary>
		/// <param name="reader">読み取りストリーム。</param>
		/// <param name="writer">書き込みストリーム。</param>
		/// <returns>Content Description Object のサイズ。</returns>
		private long SaveExtendedContentDescription( Stream reader, Stream writer )
		{
			long	sizePos  = writer.Position - 8;
			long	size     = HeaderObjectInfo.ClassByteSize + 2;	// HeaderObjectInfo とデータ数分
			ushort	newCount = 0;

			// 既存データの更新と削除
			{
				// 既存データの総数取得と仮書き込み
				ushort count = 0;
				{
					var b = reader.ReadBytes( 2 );
					writer.Write( b, 0, 2 );
					count = BitConverter.ToUInt16( b, 0 );
				}
				
				for( ushort i = 0; i < count; ++i )
				{
					var nameBytes = reader.ReadBytes( reader.ReadUInt16() );
					var name      = TagWriteData.ConvertBytesToStringUtf16( nameBytes );

					// 未定義の値なら引き継ぐ
					if( !AsfDefinedTags.HasInfo( name ) )
					{
						size += this.SaveMetadata( writer, nameBytes, reader.ReadBytes( 2 ), reader.ReadBytes( reader.ReadInt16() ) );
						++newCount;
						continue;
					}

					TagData data;
					if( this._tags.TryGetValue( name, out data ) )
					{
						if( data.IsEdited )
						{
 							// 編集内容の反映
							var v = TagWriteData.FromValue( data.Value );
							size += this.SaveMetadata( writer, nameBytes, v.TypeBytes, v.ValueBytes );
							reader.Seek( 2 + reader.ReadInt16(), SeekOrigin.Current );
						}
						else
						{
							size += this.SaveMetadata( writer, nameBytes, reader.ReadBytes( 2 ), reader.ReadBytes( reader.ReadInt16() ) );
						}

						this._tags.Remove( name );
						++newCount;
					}
					else
					{
						// 値が削除されているのでスキップ
						reader.Seek( 2 + reader.ReadInt16(), SeekOrigin.Current );
					}
				}
			}

			// 新規追加データの書き込み
			foreach( var tag in this._tags )
			{
				var v = TagWriteData.FromValue( tag.Value.Value );
				size += this.SaveMetadata( writer, TagWriteData.ConvertStringUtf16ToBytes( tag.Key ), v.TypeBytes, v.ValueBytes );
				++newCount;
			}

			// Object サイズとデータ数を更新
			{
				var pos = writer.Position;
				writer.Seek( sizePos, SeekOrigin.Begin );
				writer.Write( BitConverter.GetBytes( size ), 0, 8 );
				writer.Write( BitConverter.GetBytes( newCount ), 0, 2 );
				writer.Seek( pos, SeekOrigin.Begin );
			}

			return size;
		}
	
		/// <summary>
		/// Header Object の内容を保存します。
		/// </summary>
		/// <param name="reader">読み取りストリーム。</param>
		/// <param name="writer">書き込みストリーム。</param>
		/// <returns>ストリーム内におけるファイルサイズの更新位置。この位置から 8 バイトがファイルサイズ情報となります。</returns>
		private long SaveHeader( Stream reader, Stream writer )
		{
			// Header Object 冒頭の書き込み ( GUID ( 16 ) + Size ( 8 ) + Count ( 4 ) + Reserved ( 2 ) )
			// Header Object 全体のサイズは現時点では仮となる ( 総計を算出した後に更新する )。
			//
			int count = 0;
			{
				reader.Seek( 0, SeekOrigin.Begin );
				writer.Write( reader.ReadBytes( 30 ), 0, 30 );

				// 子オブジェクトの総数を取得
				reader.Seek( HeaderObjectInfo.ClassByteSize, SeekOrigin.Begin );
				count = reader.ReadInt32();
				reader.Seek( 2, SeekOrigin.Current );
			}

			long headerSize       = 30;
			long fileSizePosition = 0;
			{
				// Extended Content Description Object 遅延書き込み用
				long	extReaderPos = 0;
				byte[]	extInfoBytes = null;

				for( var i = 0; i < count; ++i )
				{
					// Object ヘッダを取得
					var infoBytes = reader.ReadBytes( HeaderObjectInfo.ClassByteSize );
					var info      = new HeaderObjectInfo( infoBytes, 0 );

					// Extended Content Description Object は他の固定データとタグ名が被るので、
					// それらの書き込み時にタグ情報を消して行き、余ったものを遅延書き込みする。
					// こうすることで、タグの所属をフラグで持つことなく、安全な書き込みがおこなえる。
					//
					if( info.Guid == Guids.ExtendedContentDescriptionObject )
					{
						extReaderPos = reader.Position;
						extInfoBytes = infoBytes;
						reader.Seek( info.Size - HeaderObjectInfo.ClassByteSize, SeekOrigin.Current );
						continue;
					}

					// その他の Object
					writer.Write( infoBytes, 0, infoBytes.Length );
					if( info.Guid == Guids.ContentDescriptionObject )
					{
						headerSize += this.SaveContentDescription( reader, writer, info.Size );
					}
					else
					{
						if( info.Guid == Guids.FilePropertiesObject )
						{
							// ファイルサイズ更新用の位置を記録し、この Object のタグを保存候補から除外
							fileSizePosition = writer.Position + 16;
							this._tags.Remove( AsfDefinedTags.Duration.Name );
							this._tags.Remove( AsfDefinedTags.FileSize.Name );
						}
						else if( info.Guid == Guids.StreamBitratePropertiesObject )
						{
							this._tags.Remove( AsfDefinedTags.Bitrate.Name );
						}

						headerSize += info.Size;
						this.SaveObjectData( reader, writer, info.Size - HeaderObjectInfo.ClassByteSize );
					}
				}

				// Extended Content Description Object を遅延書き込み
				var pos = reader.Position;
				if( extReaderPos != 0 )
				{
					reader.Seek( extReaderPos, SeekOrigin.Begin );
					writer.Write( extInfoBytes, 0, extInfoBytes.Length );
					headerSize += this.SaveExtendedContentDescription( reader, writer );
					reader.Seek( pos, SeekOrigin.Begin );
				}
			}

			// Header Object 全体のサイズを更新
			{
				writer.Seek( 16, SeekOrigin.Begin );
				writer.Write( BitConverter.GetBytes( headerSize ), 0, 8 );
				writer.Seek( 0, SeekOrigin.End );
			}

			return fileSizePosition;
		}
	
		/// <summary>
		/// Header Object 以降のデータを保存します。
		/// </summary>
		/// <param name="reader">読み取りストリーム。</param>
		/// <param name="writer">書き込みストリーム。</param>
		private void SaveHeaderAfter( Stream reader, Stream writer )
		{
			// Header Object の終端へ移動
			reader.Seek( 16, SeekOrigin.Begin );
			reader.Seek( reader.ReadInt64(), SeekOrigin.Begin );

			do
			{
				// Object 冒頭部分の書き込み
				long size = 0;
				{
					var b = reader.ReadBytes( HeaderObjectInfo.ClassByteSize );
					writer.Write( b, 0, b.Length );

					var info = new HeaderObjectInfo( b, 0 );
					size = info.Size - HeaderObjectInfo.ClassByteSize;
				}

				// データ部分
				this.SaveObjectData( reader, writer, size );
			}
			while( reader.Position < reader.Length );
		}

		/// <summary>
		/// メタデータを保存します。
		/// </summary>
		/// <param name="writer">書き込みストリーム。</param>
		/// <param name="name">タグ名。</param>
		/// <param name="type">データ型。</param>
		/// <param name="value">データ。</param>
		/// <returns>書き込んだデータのバイト数。</returns>
		private long SaveMetadata( Stream writer, byte[] name, byte[] type, byte[] value )
		{
			writer.Write( BitConverter.GetBytes( ( ushort )name.Length ),  0, 2            );
			writer.Write( name,                                            0, name.Length  );
			writer.Write( type,                                            0, type.Length  );
			writer.Write( BitConverter.GetBytes( ( ushort )value.Length ), 0, 2            );
			writer.Write( value,                                           0, value.Length );

			return ( 4 + name.Length + type.Length + value.Length );
		}

		/// <summary>
		/// Object のデータ部分を保存します。
		/// </summary>
		/// <param name="reader">読み取りストリーム。</param>
		/// <param name="writer">書き込みストリーム。</param>
		/// <param name="size">データ部分のサイズ。</param>
		private void SaveObjectData( Stream reader, Stream writer, long size )
		{
			// 最大 1KB 単位で読み書きする
			var bufferSize = 1024;
			do
			{
				var readSize = ( int )( size > bufferSize ? bufferSize : size );
				var b        = reader.ReadBytes( readSize );

				writer.Write( b, 0, b.Length );
				size -= b.Length;
			}
			while( size > 0 );
		}

		/// <summary>
		/// タグ情報を書き込みます。
		/// 既存のタグがなければ追加、ある場合は更新をおこないます。
		/// </summary>
		/// <param name="tag">タグ。</param>
		/// <param name="value">タグ情報。値を削除する場合は null を指定します。</param>
		/// <exception cref="NotSupportedException">サポートされていない操作をおこなおうとしました。</exception>
		/// <exception cref="ArgumentException">タグと value に指定された値の型が一致しません。</exception>
		/// <exception cref="ArgumentNullException">tag に null 参照が指定されました。</exception>
		public void Write( AsfTagInfo tag, object value )
		{
			if( tag == null ) { throw new ArgumentNullException( "tag" ); }

			// 編集の可否チェック
			if( !tag.CanEdit ) { throw new NotSupportedException( "Tag is read-only." ); }

			// 削除指定があれば終了
			if( value == null )
			{
				this._tags.Remove( tag.Name );
				this.IsEdited = true;
				return;
			}

			// 型チェック
			if( !tag.IsValidType( value ) ) { throw new ArgumentException( "Invalid value type." ); }
			
			TagData data;
			if( this._tags.TryGetValue( tag.Name, out data ) )
			{
				data.Value = value;
			}
			else
			{
				this._tags.Add( tag.Name, new TagData( value ) );
			}

			this.IsEdited = true;
		}

		/// <summary>
		/// 編集対象としている ASF ファイルのパス情報を取得します。
		/// </summary>
		public string FilePath { get; private set; }

		/// <summary>
		/// 編集がおこなわれたことを示す値を取得します。
		/// </summary>
		public bool IsEdited { get; private set; }

		/// <summary>
		/// ASF ファイルへの読み取り用ストリーム。
		/// </summary>
		private Stream _reader;

		/// <summary>
		/// タグ名をキーとする、タグ情報のディクショナリ。
		/// </summary>
		private Dictionary< string, TagData > _tags = new Dictionary< string, TagData >();
	}
}
