﻿using System.IO;

namespace Owl.Asf
{
	/// <summary>
	/// ASF のストリーム種別がビデオだった場合に定義される、ストリームの詳細情報を格納します。
	/// ASF ファイル仕様書の解説によれば、この情報は BITMAPINFOHEADER 構造体に準拠することになっています。
	/// 各フィールドの命名は ASF ファイル仕様書を踏襲しています。
	/// </summary>
	class BitmapInfoHeader
	{
		/// <summary>
		/// インスタンスを初期化します。
		/// </summary>
		public BitmapInfoHeader()
		{
		}

		/// <summary>
		/// 指定されたストリームからデータを読み取り、その内容でインスタンスを初期化します。
		/// ストリーム位置は読み取られたデータのバイト数分、進みます。
		/// </summary>
		/// <param name="stream">ストリーム。</param>
		public BitmapInfoHeader( Stream stream )
		{
			this.FormatDataSize           = stream.ReadUInt32();
			this.ImageWidth               = stream.ReadInt32();
			this.ImageHeight              = stream.ReadInt32();
			this.Reserved                 = stream.ReadUInt16();
			this.BitPerPixcelCount        = stream.ReadUInt16();
			this.CompressionId            = stream.ReadUInt32();
			this.ImageSize                = stream.ReadUInt32();
			this.HorizontalPixcelPerMeter = stream.ReadInt32();
			this.VerticalPixcelPerMeter   = stream.ReadInt32();
			this.ColorUsedCount           = stream.ReadUInt32();
			this.ImportantColorCount      = stream.ReadUInt32();
		}
		
		/// <summary>
		/// 構造体が必要とするバイト数。
		/// 構造体がカラー テーブルやカラー マスクの後に続く場合、そのカラー テーブルやカラー マスクのサイズはこの値には含まれません。
		/// </summary>
		public uint FormatDataSize;

		/// <summary>
		/// ビットマップの幅。 
		/// </summary>
		public int  ImageWidth;

		/// <summary>
		/// ビットマップの高さ。
		/// </summary>
		public int  ImageHeight;
	
		/// <summary>
		/// ターゲット デバイスに対する面の数。必ず 1 に設定されます。
		/// </summary>
		public ushort Reserved;
		
		/// <summary>
		/// 1 ピクセルあたりのビット数。
		/// </summary>
		public ushort BitPerPixcelCount;
	
		/// <summary>
		/// 圧縮方法を示す 4 文字 ( ASCII ) の識別子。
		/// ISO MPEG-4 ビデオの場合は "MP4S"、"mp4s"、"M4S2"、または "m4s2" が指定されます。
		/// 識別子の文字は下位バイトを先頭に上位バイトへ向かって並んでいます。
		/// 例えば MP4S の場合は 0x5334504D、つまり { 'S' = 53, '4' = 34, 'P' = 50, 'M' = 4D } となります。
		/// </summary>
		public uint CompressionId;

		/// <summary>
		/// イメージのサイズを指定する ( バイト単位 )。
		/// 非圧縮 RGB ビットマップの場合は、0 に設定できます。
		/// </summary>
		public uint ImageSize;

		/// <summary>
		/// ビットマップのターゲット デバイスの水平解像度を表す 1 メートルあたりのピクセル単位。
		/// </summary>
		public int HorizontalPixcelPerMeter;

		/// <summary>
		/// ビットマップのターゲット デバイスの垂直解像度を表す 1 メートルあたりのピクセル単位。 
		/// </summary>
		public int  VerticalPixcelPerMeter;

		/// <summary>
		/// カラー テーブル内のカラー インデックスのうち、ビットマップ内で実際に使うインデックスの数。
		/// </summary>
		public uint ColorUsedCount;

		/// <summary>
		/// ビットマップを表示するために重要と見なされるカラー インデックス数。
		/// この値がゼロの場合は、すべての色が重要と見なされます。
		/// </summary>
		public uint ImportantColorCount;
	}
}
