﻿using System;
using System.Text;

namespace Owl.Asf
{
	/// <summary>
	/// ファイルに書き込むためのデータを格納します。
	/// </summary>
	class TagWriteData
	{
		/// <summary>
		/// インスタンスを初期化します。
		/// </summary>
		/// <param name="type">データ種別。</param>
		/// <param name="value">データを示すバイト配列。</param>
		public TagWriteData( TagDataType type, byte[] value )
		{
			this.TypeBytes  = BitConverter.GetBytes( ( ushort )type );
			this.ValueBytes = value;
		}

		/// <summary>
		/// バイト配列を UTF-16LE 文字列に変換します。
		/// </summary>
		/// <param name="bytes">バイト配列。</param>
		/// <returns>変換した文字列。</returns>
		public static string ConvertBytesToStringUtf16( byte[] bytes )
		{
			// 文字列化と終端の NULL 文字を除去
			var str = Encoding.Unicode.GetString( bytes, 0, bytes.Length );
			if( str[ str.Length - 1 ] == '\0' )
			{
				if( str.Length == 1 ) { return String.Empty; }

				var sb = new StringBuilder( str );
				sb.Remove( sb.Length - 1, 1 );
				str = sb.ToString();
			}

			return str;
		}

		/// <summary>
		/// UTF-16LE 文字列を NULL 文字終端のバイト配列に変換します。
		/// </summary>
		/// <param name="text">文字列</param>
		/// <returns></returns>
		public static byte[] ConvertStringUtf16ToBytes( string text )
		{
			var sb = new StringBuilder( text );
			sb.Append( '\0' );
			return Encoding.Unicode.GetBytes( sb.ToString() );
		}

		/// <summary>
		/// データから TagWriteValue インスタンスを生成します。
		/// </summary>
		/// <param name="value">データ。</param>
		/// <returns>成功時は WriteValue インスタンス。失敗時は null 参照。</returns>
		public static TagWriteData FromValue( object value )
		{
			if( value is string )
			{
				return new TagWriteData( TagDataType.String, TagWriteData.ConvertStringUtf16ToBytes( ( string )value ) );
			}
			else if( value is bool )
			{
				// WMT_TYPE_BOOL は 4 バイトなので uint に変換する
				var v = ( uint )( ( bool )value ? 1 : 0 );
				return new TagWriteData( TagDataType.Bool, BitConverter.GetBytes( v ) );
			}
			else if( value is ushort )
			{
				return new TagWriteData( TagDataType.UInt16, BitConverter.GetBytes( ( ushort )value ) );
			}
			else if( value is uint )
			{
				return new TagWriteData( TagDataType.UInt32, BitConverter.GetBytes( ( uint )value ) );
			}
			else if( value is ulong )
			{
				return new TagWriteData( TagDataType.UInt64, BitConverter.GetBytes( ( ulong )value ) );
			}
			else if( value is Guid )
			{
				return new TagWriteData( TagDataType.String, ( ( Guid )value ).ToByteArray() );
			}
			else if( value is byte[] )
			{
				return new TagWriteData( TagDataType.Binary, ( byte[] )value );
			}

			return null;
		}

		/// <summary>
		/// データ種別を示すバイト配列取得します。
		/// </summary>
		public byte[] TypeBytes { get; private set; }

		/// <summary>
		/// データを示すバイト配列を取得します。
		/// </summary>
		public byte[] ValueBytes { get; private set; }
	}
}
