﻿using System.Collections.Generic;

namespace Owl.Id3
{
	/// <summary>
	/// 定義済みのタグ ( ID3v2.2 ～ 2.4 ) に関する情報を提供します。
	/// </summary>
	public static class Id3DefinedTags
	{
		/// <summary>
		/// 定義済みのタグ ( ID3v2.2 ) に関する情報を提供します。
		/// </summary>
		public static class V22
		{
			/// <summary>
			/// 指定された名前を持つタグ情報を取得します。
			/// </summary>
			/// <param name="id">フレーム ID。</param>
			/// <returns>成功時はタグ情報。それ以外は null 参照。</returns>
			internal static Id3TagInfo GetInfo( string id )
			{
				Id3TagInfo info;
				return ( V22.Infos.TryGetValue( id, out info ) ? info : null );
			}

			/// <summary>
			/// タグ情報のコレクションを取得します。
			/// </summary>
			/// <returns>タグ情報のコレクション。</returns>
			public static IEnumerable< Id3TagInfo > GetTags()
			{
				return V22.Infos.Values;
			}

			/// <summary>
			/// 指定された名前を持つタグが定義されていることを調べます。
			/// </summary>
			/// <param name="id">フレーム ID。</param>
			/// <returns>定義されている場合は true。それ以外は false。</returns>
			internal static bool HasInfo( string id )
			{
				return V22.Infos.ContainsKey( id );
			}

			/// <summary>
			/// 推奨バッファサイズを示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo BUF = new Id3TagInfo( "BUF", Id3TagDataType.Binary );

			/// <summary>
			/// 再生回数を示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo CNT = new Id3TagInfo( "CNT", Id3TagDataType.Binary );

			/// <summary>
			/// コメントを示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo COM = new Id3TagInfo( "COM", Id3TagDataType.TextWithCommentAndLanguage );

			/// <summary>
			/// オーディオの暗号化形式を示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo CRA = new Id3TagInfo( "CRA", Id3TagDataType.BinaryWithComment );

			/// <summary>
			/// 暗号化メタフレームを示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo CRM = new Id3TagInfo( "CRM", Id3TagDataType.Binary );

			/// <summary>
			/// イベントタイムコードを示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo ETC = new Id3TagInfo( "ETC", Id3TagDataType.Binary );

			/// <summary>
			/// イコライザー情報を示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo EQU = new Id3TagInfo( "EQU", Id3TagDataType.Binary );

			/// <summary>
			/// パッケージ化された一般的なオブジェクトを示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo GEO = new Id3TagInfo( "GEO", Id3TagDataType.Binary );

			/// <summary>
			/// 協力者を示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo IPL = new Id3TagInfo( "IPL", Id3TagDataType.Text );

			/// <summary>
			/// リンク情報を示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo LNK = new Id3TagInfo( "LNK", Id3TagDataType.Link );

			/// <summary>
			/// 音楽 CD の識別子を示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo MCI = new Id3TagInfo( "MCI", Id3TagDataType.Binary );

			/// <summary>
			/// MPEG ロケーションルックアップテーブルを示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo MLL = new Id3TagInfo( "MLL", Id3TagDataType.Binary );

			/// <summary>
			/// ジャケット画像を示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo PIC = new Id3TagInfo( "PIC", Id3TagDataType.Binary );

			/// <summary>
			/// 人気 ( 1 ～ 255 の数値で表す評価 ) を示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo POP = new Id3TagInfo( "POP", Id3TagDataType.BinaryWithComment );

			/// <summary>
			/// リバーブ量を示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo REV = new Id3TagInfo( "REV", Id3TagDataType.Binary );

			/// <summary>
			/// 相対ボリュームを示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo RVA = new Id3TagInfo( "RVA", Id3TagDataType.Binary );

			/// <summary>
			/// タイムラインを持つ歌詞情報を示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo SLT = new Id3TagInfo( "SLT", Id3TagDataType.Binary );

			/// <summary>
			/// タイムラインを持つテンポ ( Bit Per Minute ) 情報を示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo STC = new Id3TagInfo( "STC", Id3TagDataType.Binary );

			/// <summary>
			/// 曲の属するアルバム、映画、劇などの名前を示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo TAL = new Id3TagInfo( "TAL", Id3TagDataType.Text );

			/// <summary>
			/// テンポ ( Bit Per Minute ) を示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo TBP = new Id3TagInfo( "TBP", Id3TagDataType.Text );

			/// <summary>
			/// 作曲者を示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo TCM = new Id3TagInfo( "TCM", Id3TagDataType.Text );

			/// <summary>
			/// ジャンルを示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo TCO = new Id3TagInfo( "TCO", Id3TagDataType.Text );

			/// <summary>
			/// 著作権情報を示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo TCR = new Id3TagInfo( "TCR", Id3TagDataType.Text );
	
			/// <summary>
			/// 録音された日付を示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo TDA = new Id3TagInfo( "TDA", Id3TagDataType.Text );

			/// <summary>
			/// プレイリストの遅延時間 ( 曲間の無音時間をスキップするための情報 ) を示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo TDY = new Id3TagInfo( "TDY", Id3TagDataType.Text );

			/// <summary>
			/// オーディオファイルの形式を示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo TEN = new Id3TagInfo( "TEN", Id3TagDataType.Text );

			/// <summary>
			/// ファイルのエンコードをおこなった個人または組織を示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo TFT = new Id3TagInfo( "TFT", Id3TagDataType.Text );

			/// <summary>
			/// レコーディングに要した時間 ( 形式は HHMM、計 4 桁の文字列 ) を示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo TIM = new Id3TagInfo( "TIM", Id3TagDataType.Text );

			/// <summary>
			/// 曲頭のキー ( C、Am、F♯ といった調の文字列 ) を示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo TKE = new Id3TagInfo( "TKE", Id3TagDataType.Text );

			/// <summary>
			/// 言語 ( ISO-639-2 として定義された 3 文字の文字列 ) を示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo TLA = new Id3TagInfo( "TLA", Id3TagDataType.Text );

			/// <summary>
			/// 演奏時間 ( ミリ秒単位 ) を示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo TLE = new Id3TagInfo( "TLE", Id3TagDataType.Text );

			/// <summary>
			/// メディア種別 ( 音楽 CD、レコード、...etc ) を示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo TMT = new Id3TagInfo( "TMT", Id3TagDataType.Text );

			/// <summary>
			/// 原曲のアーティストを示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo TOA = new Id3TagInfo( "TOA", Id3TagDataType.Text );

			/// <summary>
			/// 元のファイル名を示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo TOF = new Id3TagInfo( "TOF", Id3TagDataType.Text );

			/// <summary>
			/// 原曲の作詞者を示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo TOL = new Id3TagInfo( "TOL", Id3TagDataType.Text );

			/// <summary>
			/// 原曲のリリース年を示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo TOR = new Id3TagInfo( "TOR", Id3TagDataType.Text );

			/// <summary>
			/// 原曲の属するアルバム、映画、劇などの名前を示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo TOT = new Id3TagInfo( "TOT", Id3TagDataType.Text );

			/// <summary>
			/// 主なアーティスト、演奏者などを示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo TP1 = new Id3TagInfo( "TP1", Id3TagDataType.Text );

			/// <summary>
			/// レコーディングをおこなった演奏者の追加情報を示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo TP2 = new Id3TagInfo( "TP2", Id3TagDataType.Text );

			/// <summary>
			/// 指揮者を示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo TP3 = new Id3TagInfo( "TP3", Id3TagDataType.Text );

			/// <summary>
			/// リミックスや編曲の担当者を示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo TP4 = new Id3TagInfo( "TP4", Id3TagDataType.Text );

			/// <summary>
			/// 曲の属するグループ内の番号 ( スラッシュ区切りの文字列 ) を示すタグ定義を取得します。
			/// 複数枚や複数の組曲で構成されるアルバムの場合、それらの総数と何番目に位置するかを表します。
			/// 例えば 3 枚組のアルバムにおいて、2 枚目に属する曲ならば "2/3" となります。
			/// </summary>
			public static readonly Id3TagInfo TPA = new Id3TagInfo( "TPA", Id3TagDataType.Text );

			/// <summary>
			/// 曲の発行元 ( レーベル ) を示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo TPB = new Id3TagInfo( "TPB", Id3TagDataType.Text );

			/// <summary>
			/// 国際標準レコーディングコード（ International Standard Recording Code ) を示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo TRC = new Id3TagInfo( "TRC", Id3TagDataType.Text );

			/// <summary>
			/// 録音日時を示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo TRD = new Id3TagInfo( "TRD", Id3TagDataType.Text );

			/// <summary>
			/// トラック番号 ( 書式は "番号/総数" ) を示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo TRK = new Id3TagInfo( "TRK", Id3TagDataType.Text );

			/// <summary>
			/// オーディオファイルのサイズ ( ID3v2 タグを除いたバイト数 ) を示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo TSI = new Id3TagInfo( "TSI", Id3TagDataType.Text );

			/// <summary>
			/// エンコード設定を示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo TSS = new Id3TagInfo( "TSS", Id3TagDataType.Text );

			/// <summary>
			/// グループの説明を示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo TT1 = new Id3TagInfo( "TT1", Id3TagDataType.Text );

			/// <summary>
			/// タイトルを示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo TT2 = new Id3TagInfo( "TT2", Id3TagDataType.Text );

			/// <summary>
			/// 副題やタイトルの補足説明を示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo TT3 = new Id3TagInfo( "TT3", Id3TagDataType.Binary );

			/// <summary>
			/// 作詞家を示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo TXT = new Id3TagInfo( "TXT", Id3TagDataType.Text );

			/// <summary>
			/// ユーザー定義文字情報フレームを示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo TXX = new Id3TagInfo( "TXX", Id3TagDataType.TextUserDefined );

			/// <summary>
			/// リリース年を示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo TYE = new Id3TagInfo( "TYE", Id3TagDataType.Text );

			/// <summary>
			/// ファイルの一意な識別子を示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo UFI = new Id3TagInfo( "UFI", Id3TagDataType.Binary );

			/// <summary>
			/// 歌詞を示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo ULT = new Id3TagInfo( "ULT", Id3TagDataType.TextWithCommentAndLanguage );

			/// <summary>
			/// オーディオファイルの公式サイト URL を示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo WAF = new Id3TagInfo( "WAF", Id3TagDataType.Url );

			/// <summary>
			/// アーティストの公式サイト URL を示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo WAR = new Id3TagInfo( "WAR", Id3TagDataType.Url );

			/// <summary>
			/// 音源の公式サイト URL を示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo WAS = new Id3TagInfo( "WAS", Id3TagDataType.Url );

			/// <summary>
			/// 販売サイト URL を示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo WCM = new Id3TagInfo( "WCM", Id3TagDataType.Url );

			/// <summary>
			/// 著作情報サイト URL を示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo WCP = new Id3TagInfo( "WCP", Id3TagDataType.Url );

			/// <summary>
			/// レーベルの公式サイト URL を示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo WPB = new Id3TagInfo( "WPB", Id3TagDataType.Url );

			/// <summary>
			/// ユーザー定義 URL を示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo WXX = new Id3TagInfo( "WXX", Id3TagDataType.Binary );

			/// <summary>
			/// フレーム ID をキーとするタグ情報のディクショナリ。
			/// </summary>
			private static Dictionary< string, Id3TagInfo > Infos = new Dictionary< string, Id3TagInfo >()
			{
				{ V22.BUF.Id, V22.BUF },
				{ V22.CNT.Id, V22.CNT },
				{ V22.COM.Id, V22.COM },
				{ V22.CRA.Id, V22.CRA },
				{ V22.CRM.Id, V22.CRM },
				{ V22.ETC.Id, V22.ETC },
				{ V22.EQU.Id, V22.EQU },
				{ V22.GEO.Id, V22.GEO },
				{ V22.IPL.Id, V22.IPL },
				{ V22.LNK.Id, V22.LNK },
				{ V22.MCI.Id, V22.MCI },
				{ V22.MLL.Id, V22.MLL },
				{ V22.PIC.Id, V22.PIC },
				{ V22.POP.Id, V22.POP },
				{ V22.REV.Id, V22.REV },
				{ V22.RVA.Id, V22.RVA },
				{ V22.SLT.Id, V22.SLT },
				{ V22.STC.Id, V22.STC },
				{ V22.TAL.Id, V22.TAL },
				{ V22.TBP.Id, V22.TBP },
				{ V22.TCM.Id, V22.TCM },
				{ V22.TCO.Id, V22.TCO },
				{ V22.TCR.Id, V22.TCR },
				{ V22.TDA.Id, V22.TDA },
				{ V22.TDY.Id, V22.TDY },
				{ V22.TEN.Id, V22.TEN },
				{ V22.TFT.Id, V22.TFT },
				{ V22.TIM.Id, V22.TIM },
				{ V22.TKE.Id, V22.TKE },
				{ V22.TLA.Id, V22.TLA },
				{ V22.TLE.Id, V22.TLE },
				{ V22.TMT.Id, V22.TMT },
				{ V22.TOA.Id, V22.TOA },
				{ V22.TOF.Id, V22.TOF },
				{ V22.TOL.Id, V22.TOL },
				{ V22.TOR.Id, V22.TOR },
				{ V22.TOT.Id, V22.TOT },
				{ V22.TP1.Id, V22.TP1 },
				{ V22.TP2.Id, V22.TP2 },
				{ V22.TP3.Id, V22.TP3 },
				{ V22.TP4.Id, V22.TP4 },
				{ V22.TPA.Id, V22.TPA },
				{ V22.TPB.Id, V22.TPB },
				{ V22.TRC.Id, V22.TRC },
				{ V22.TRD.Id, V22.TRD },
				{ V22.TRK.Id, V22.TRK },
				{ V22.TSI.Id, V22.TSI },
				{ V22.TSS.Id, V22.TSS },
				{ V22.TT1.Id, V22.TT1 },
				{ V22.TT2.Id, V22.TT2 },
				{ V22.TT3.Id, V22.TT3 },
				{ V22.TXT.Id, V22.TXT },
				{ V22.TXX.Id, V22.TXX },
				{ V22.TYE.Id, V22.TYE },
				{ V22.UFI.Id, V22.UFI },
				{ V22.ULT.Id, V22.ULT },
				{ V22.WAF.Id, V22.WAF },
				{ V22.WAR.Id, V22.WAR },
				{ V22.WAS.Id, V22.WAS },
				{ V22.WCM.Id, V22.WCM },
				{ V22.WCP.Id, V22.WCP },
				{ V22.WPB.Id, V22.WPB },
				{ V22.WXX.Id, V22.WXX }
			};
		}

		/// <summary>
		/// 定義済みのタグ ( ID3v2.3 ) に関する情報を提供します。
		/// </summary>
		public class V23
		{	
			/// <summary>
			/// 指定された名前を持つタグ情報を取得します。
			/// </summary>
			/// <param name="id">フレーム ID。</param>
			/// <returns>成功時はタグ情報。それ以外は null 参照。</returns>
			internal static Id3TagInfo GetInfo( string id )
			{
				Id3TagInfo info;
				return ( V23.Infos.TryGetValue( id, out info ) ? info : null );
			}

			/// <summary>
			/// タグ情報のコレクションを取得します。
			/// </summary>
			/// <returns>タグ情報のコレクション。</returns>
			public static IEnumerable< Id3TagInfo > GetTags()
			{
				return V23.Infos.Values;
			}

			/// <summary>
			/// 指定された名前を持つタグが定義されていることを調べます。
			/// </summary>
			/// <param name="id">フレーム ID。</param>
			/// <returns>定義されている場合は true。それ以外は false。</returns>
			internal static bool HasInfo( string id )
			{
				return V23.Infos.ContainsKey( id );
			}
			/// <summary>
			/// オーディオの暗号化形式を示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo ANEC = new Id3TagInfo( "ANEC", Id3TagDataType.BinaryWithComment );
	
			/// <summary>
			/// ジャケット画像を示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo APIC = new Id3TagInfo( "APIC", Id3TagDataType.Binary );

			/// <summary>
			/// コメントを示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo COMM = new Id3TagInfo( "COMM", Id3TagDataType.TextWithCommentAndLanguage );

			/// <summary>
			/// コマーシャル フレームを示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo COMR = new Id3TagInfo( "COMR", Id3TagDataType.Binary );

			/// <summary>
			/// 暗号化手法を示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo ENCR = new Id3TagInfo( "ENCR", Id3TagDataType.BinaryWithComment );

			/// <summary>
			/// イコライザー情報を示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo EQUA = new Id3TagInfo( "EQUA", Id3TagDataType.Binary );

			/// <summary>
			/// イベント タイム コードを示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo ETCO = new Id3TagInfo( "ETCO", Id3TagDataType.Binary );

			/// <summary>
			/// パッケージ化された一般的なオブジェクトを示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo GEOB = new Id3TagInfo( "GEOB", Id3TagDataType.Binary );

			/// <summary>
			/// グループ識別子を示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo GRID = new Id3TagInfo( "GRID", Id3TagDataType.BinaryWithComment );

			/// <summary>
			/// 協力者を示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo IPLS = new Id3TagInfo( "IPLS", Id3TagDataType.Text );

			/// <summary>
			/// リンク情報を示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo LINK = new Id3TagInfo( "LINK", Id3TagDataType.Link );

			/// <summary>
			/// 音楽 CD の識別子を示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo MCDI = new Id3TagInfo( "MCDI", Id3TagDataType.Binary );

			/// <summary>
			/// MPEG ロケーション ルックアップ テーブルを示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo MLLT = new Id3TagInfo( "MLLT", Id3TagDataType.Binary );

			/// <summary>
			/// 所有権フレームを示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo OWNE = new Id3TagInfo( "OWNE", Id3TagDataType.Binary );

			/// <summary>
			/// プライベート フレームを示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo PRIV = new Id3TagInfo( "PRIV", Id3TagDataType.BinaryWithComment );

			/// <summary>
			/// 再生回数を示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo PCNT = new Id3TagInfo( "PCNT", Id3TagDataType.Binary );

			/// <summary>
			/// 人気 ( 1 ～ 255 の数値で表す評価 ) を示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo POPM = new Id3TagInfo( "POPM", Id3TagDataType.BinaryWithComment );

			/// <summary>
			/// 同期位置フレームを示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo POSS = new Id3TagInfo( "POSS", Id3TagDataType.Binary );

			/// <summary>
			/// 推奨バッファサイズを示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo RBUF = new Id3TagInfo( "RBUF", Id3TagDataType.Binary );

			/// <summary>
			/// 相対ボリュームを示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo RVAD = new Id3TagInfo( "RVAD", Id3TagDataType.Binary );

			/// <summary>
			/// リバーブ量を示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo RVRB = new Id3TagInfo( "RVRB", Id3TagDataType.Binary );

			/// <summary>
			/// タイムラインを持つ歌詞情報を示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo SYLT = new Id3TagInfo( "SYLT", Id3TagDataType.Binary );

			/// <summary>
			/// タイムラインを持つテンポ ( Bit Per Minute ) 情報を示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo SYTC = new Id3TagInfo( "SYTC", Id3TagDataType.Binary );

			/// <summary>
			/// 曲の属するアルバム、映画、劇などの名前を示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo TALB = new Id3TagInfo( "TALB", Id3TagDataType.Text );

			/// <summary>
			/// テンポ ( Bit Per Minute ) を示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo TBPM = new Id3TagInfo( "TBPM", Id3TagDataType.Text );

			/// <summary>
			/// 作曲者を示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo TCOM = new Id3TagInfo( "TCOM", Id3TagDataType.Text );

			/// <summary>
			/// ジャンルを示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo TCON = new Id3TagInfo( "TCON", Id3TagDataType.Text );

			/// <summary>
			/// 著作権情報を示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo TCOP = new Id3TagInfo( "TCOP", Id3TagDataType.Text );

			/// <summary>
			/// 録音された日付を示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo TDAT = new Id3TagInfo( "TDAT", Id3TagDataType.Text );

			/// <summary>
			/// プレイリストの遅延時間 ( 曲間の無音時間をスキップするための情報 ) を示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo TDLY = new Id3TagInfo( "TDLY", Id3TagDataType.Text );

			/// <summary>
			/// ファイルのエンコードをおこなった個人または組織を示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo TENC = new Id3TagInfo( "TENC", Id3TagDataType.Text );

			/// <summary>
			/// 作詞家を示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo TEXT = new Id3TagInfo( "TEXT", Id3TagDataType.Text );

			/// <summary>
			/// オーディオファイルの形式を示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo TFLT = new Id3TagInfo( "TFLT", Id3TagDataType.Text );

			/// <summary>
			/// レコーディングに要した時間 ( 形式は HHMM、計 4 桁の文字列 ) を示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo TIME = new Id3TagInfo( "TIME", Id3TagDataType.Text );

			/// <summary>
			/// グループの説明を示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo TIT1 = new Id3TagInfo( "TIT1", Id3TagDataType.Text );

			/// <summary>
			/// タイトルを示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo TIT2 = new Id3TagInfo( "TIT2", Id3TagDataType.Text );

			/// <summary>
			/// 副題やタイトルの補足説明を示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo TIT3 = new Id3TagInfo( "TIT3", Id3TagDataType.Text );

			/// <summary>
			/// 曲頭のキー ( C、Am、F♯ といった調の文字列 ) を示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo TKEY = new Id3TagInfo( "TKEY", Id3TagDataType.Text );

			/// <summary>
			/// 言語 ( ISO-639-2 として定義された 3 文字の文字列 ) を示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo TLAN = new Id3TagInfo( "TLAN", Id3TagDataType.Text );

			/// <summary>
			/// 演奏時間 ( ミリ秒単位 ) を示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo TLEN = new Id3TagInfo( "TLEN", Id3TagDataType.Text );

			/// <summary>
			/// メディア種別 ( 音楽 CD、レコード、...etc ) を示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo TMED = new Id3TagInfo( "TMED", Id3TagDataType.Text );

			/// <summary>
			/// 原曲の属するアルバム、映画、劇などの名前を示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo TOAL = new Id3TagInfo( "TOAL", Id3TagDataType.Text );

			/// <summary>
			/// 元のファイル名を示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo TOFN = new Id3TagInfo( "TOFN", Id3TagDataType.Text );

			/// <summary>
			/// 原曲の作詞者を示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo TOLY = new Id3TagInfo( "TOLY", Id3TagDataType.Text );

			/// <summary>
			/// 原曲のアーティストを示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo TOPE = new Id3TagInfo( "TOPE", Id3TagDataType.Text );

			/// <summary>
			/// 原曲のリリース年を示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo TORY = new Id3TagInfo( "TORY", Id3TagDataType.Text );

			/// <summary>
			/// ファイルの所有者・ライセンシーを示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo TOWN = new Id3TagInfo( "TOWN", Id3TagDataType.Text );

			/// <summary>
			/// 主なアーティスト、演奏者などを示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo TPE1 = new Id3TagInfo( "TPE1", Id3TagDataType.Text );

			/// <summary>
			/// レコーディングをおこなった演奏者の追加情報を示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo TPE2 = new Id3TagInfo( "TPE2", Id3TagDataType.Text );

			/// <summary>
			/// 指揮者を示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo TPE3 = new Id3TagInfo( "TPE3", Id3TagDataType.Text );

			/// <summary>
			/// リミックスや編曲の担当者を示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo TPE4 = new Id3TagInfo( "TPE4", Id3TagDataType.Text );

			/// <summary>
			/// 曲の属するグループ内の番号 ( スラッシュ区切りの文字列 ) を示すタグ定義を取得します。
			/// 複数枚や複数の組曲で構成されるアルバムの場合、それらの総数と何番目に位置するかを表します。
			/// 例えば 3 枚組のアルバムにおいて、2 枚目に属する曲ならば "2/3" となります。
			/// </summary>
			public static readonly Id3TagInfo TPOS = new Id3TagInfo( "TPOS", Id3TagDataType.Text );

			/// <summary>
			/// 曲の発行元 ( レーベル ) を示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo TPUB = new Id3TagInfo( "TPUB", Id3TagDataType.Text );

			/// <summary>
			/// トラック番号 ( 書式は "番号/総数" ) を示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo TRCK = new Id3TagInfo( "TRCK", Id3TagDataType.Text );

			/// <summary>
			/// 録音日時を示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo TRDA = new Id3TagInfo( "TRDA", Id3TagDataType.Text );

			/// <summary>
			/// インターネットラジオ局を示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo TRSN = new Id3TagInfo( "TRSN", Id3TagDataType.Text );

			/// <summary>
			/// インターネットラジオ局の所有者を示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo TRSO = new Id3TagInfo( "TRSO", Id3TagDataType.Text );

			/// <summary>
			/// オーディオファイルのサイズ ( ID3v2 タグを除いたバイト数 ) を示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo TSIZ = new Id3TagInfo( "TSIZ", Id3TagDataType.Text );

			/// <summary>
			/// 国際標準レコーディングコード（ International Standard Recording Code ) を示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo TSRC = new Id3TagInfo( "TSRC", Id3TagDataType.Text );

			/// <summary>
			/// エンコード設定を示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo TSSE = new Id3TagInfo( "TSSE", Id3TagDataType.Text );

			/// <summary>
			/// リリース年を示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo TYER = new Id3TagInfo( "TYER", Id3TagDataType.Text );

			/// <summary>
			/// ユーザー定義文字情報フレームを示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo TXXX = new Id3TagInfo( "TXXX", Id3TagDataType.TextUserDefined );

			/// <summary>
			/// ファイルの一意な識別子を示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo UFID = new Id3TagInfo( "UFID", Id3TagDataType.BinaryWithComment );

			/// <summary>
			/// 使用条件を示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo USER = new Id3TagInfo( "USER", Id3TagDataType.TextWithLanguage );

			/// <summary>
			/// 歌詞を示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo USLT = new Id3TagInfo( "USLT", Id3TagDataType.TextWithCommentAndLanguage );

			/// <summary>
			/// 販売サイト URL を示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo WCOM = new Id3TagInfo( "WCOM", Id3TagDataType.Url );

			/// <summary>
			/// 著作情報サイト URL を示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo WCOP = new Id3TagInfo( "WCOP", Id3TagDataType.Url );

			/// <summary>
			/// オーディオファイルの公式サイト URL を示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo WOAF = new Id3TagInfo( "WOAF", Id3TagDataType.Url );

			/// <summary>
			/// アーティストの公式サイト URL を示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo WOAR = new Id3TagInfo( "WOAR", Id3TagDataType.Url );

			/// <summary>
			/// 音源の公式サイト URL を示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo WOAS = new Id3TagInfo( "WOAS", Id3TagDataType.Url );

			/// <summary>
			/// インターネットラジオ局の公式サイト URL を示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo WORS = new Id3TagInfo( "WORS", Id3TagDataType.Url );

			/// <summary>
			/// 支払い方法を示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo WPAY = new Id3TagInfo( "WPAY", Id3TagDataType.Url );

			/// <summary>
			/// レーベルの公式サイト URL を示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo WPUB = new Id3TagInfo( "WPUB", Id3TagDataType.Url );

			/// <summary>
			/// ユーザー定義 URL を示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo WXXX = new Id3TagInfo( "WXXX", Id3TagDataType.TextUserDefined );	


			/// <summary>
			/// フレーム ID をキーとするタグ情報のディクショナリ。
			/// </summary>
			private static readonly Dictionary< string, Id3TagInfo > Infos = new Dictionary< string, Id3TagInfo >()
			{
				{ V23.ANEC.Id, V23.ANEC },
				{ V23.APIC.Id, V23.APIC },
				{ V23.COMM.Id, V23.COMM },
				{ V23.COMR.Id, V23.COMR },
				{ V23.ENCR.Id, V23.ENCR },
				{ V23.EQUA.Id, V23.EQUA },
				{ V23.ETCO.Id, V23.ETCO },
				{ V23.GEOB.Id, V23.GEOB },
				{ V23.GRID.Id, V23.GRID },
				{ V23.IPLS.Id, V23.IPLS },
				{ V23.LINK.Id, V23.LINK },
				{ V23.MCDI.Id, V23.MCDI },
				{ V23.MLLT.Id, V23.MLLT },
				{ V23.OWNE.Id, V23.OWNE },
				{ V23.PRIV.Id, V23.PRIV },
				{ V23.PCNT.Id, V23.PCNT },
				{ V23.POPM.Id, V23.POPM },
				{ V23.POSS.Id, V23.POSS },
				{ V23.RBUF.Id, V23.RBUF },
				{ V23.RVAD.Id, V23.RVAD },
				{ V23.RVRB.Id, V23.RVRB },
				{ V23.SYLT.Id, V23.SYLT },
				{ V23.SYTC.Id, V23.SYTC },
				{ V23.TALB.Id, V23.TALB },
				{ V23.TBPM.Id, V23.TBPM },
				{ V23.TCOM.Id, V23.TCOM },
				{ V23.TCON.Id, V23.TCON },
				{ V23.TCOP.Id, V23.TCOP },
				{ V23.TDAT.Id, V23.TDAT },
				{ V23.TDLY.Id, V23.TDLY },
				{ V23.TENC.Id, V23.TENC },
				{ V23.TEXT.Id, V23.TEXT },
				{ V23.TFLT.Id, V23.TFLT },
				{ V23.TIME.Id, V23.TIME },
				{ V23.TIT1.Id, V23.TIT1 },
				{ V23.TIT2.Id, V23.TIT2 },
				{ V23.TIT3.Id, V23.TIT3 },
				{ V23.TKEY.Id, V23.TKEY },
				{ V23.TLAN.Id, V23.TLAN },
				{ V23.TLEN.Id, V23.TLEN },
				{ V23.TMED.Id, V23.TMED },
				{ V23.TOAL.Id, V23.TOAL },
				{ V23.TOFN.Id, V23.TOFN },
				{ V23.TOLY.Id, V23.TOLY },
				{ V23.TOPE.Id, V23.TOPE },
				{ V23.TORY.Id, V23.TORY },
				{ V23.TOWN.Id, V23.TOWN },
				{ V23.TPE1.Id, V23.TPE1 },
				{ V23.TPE2.Id, V23.TPE2 },
				{ V23.TPE3.Id, V23.TPE3 },
				{ V23.TPE4.Id, V23.TPE4 },
				{ V23.TPOS.Id, V23.TPOS },
				{ V23.TPUB.Id, V23.TPUB },
				{ V23.TRCK.Id, V23.TRCK },
				{ V23.TRDA.Id, V23.TRDA },
				{ V23.TRSN.Id, V23.TRSN },
				{ V23.TRSO.Id, V23.TRSO },
				{ V23.TSIZ.Id, V23.TSIZ },
				{ V23.TSRC.Id, V23.TSRC },
				{ V23.TSSE.Id, V23.TSSE },
				{ V23.TYER.Id, V23.TYER },
				{ V23.TXXX.Id, V23.TXXX },
				{ V23.UFID.Id, V23.UFID },
				{ V23.USER.Id, V23.USER },
				{ V23.USLT.Id, V23.USLT },
				{ V23.WCOM.Id, V23.WCOM },
				{ V23.WCOP.Id, V23.WCOP },
				{ V23.WOAF.Id, V23.WOAF },
				{ V23.WOAR.Id, V23.WOAR },
				{ V23.WOAS.Id, V23.WOAS },
				{ V23.WORS.Id, V23.WORS },
				{ V23.WPAY.Id, V23.WPAY },
				{ V23.WPUB.Id, V23.WPUB },
				{ V23.WXXX.Id, V23.WXXX }
			};
		}

		/// <summary>
		/// 定義済みのタグ ( ID3v2.4 ) に関する情報を提供します。
		/// </summary>
		public class V24
		{		
			/// <summary>
			/// 指定された名前を持つタグ情報を取得します。
			/// </summary>
			/// <param name="id">フレーム ID。</param>
			/// <returns>成功時はタグ情報。それ以外は null 参照。</returns>
			internal static Id3TagInfo GetInfo( string id )
			{
				Id3TagInfo info;
				return ( V24.Infos.TryGetValue( id, out info ) ? info : null );
			}

			/// <summary>
			/// タグ情報のコレクションを取得します。
			/// </summary>
			/// <returns>タグ情報のコレクション。</returns>
			public static IEnumerable< Id3TagInfo > GetTags()
			{
				return V24.Infos.Values;
			}

			/// <summary>
			/// 指定された名前を持つタグが定義されていることを調べます。
			/// </summary>
			/// <param name="id">フレーム ID。</param>
			/// <returns>定義されている場合は true。それ以外は false。</returns>
			internal static bool HasInfo( string id )
			{
				return V24.Infos.ContainsKey( id );
			}

			/// <summary>
			/// オーディオの暗号化形式を示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo ANEC = new Id3TagInfo( "ANEC", Id3TagDataType.BinaryWithComment );

			/// <summary>
			/// ジャケット画像を示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo APIC = new Id3TagInfo( "APIC", Id3TagDataType.Binary );

			/// <summary>
			/// オーディオ シーケンス ポイント インデックスを示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo ASPI = new Id3TagInfo( "ASPI", Id3TagDataType.Binary );

			/// <summary>
			/// コメントを示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo COMM = new Id3TagInfo( "COMM", Id3TagDataType.TextWithCommentAndLanguage );

			/// <summary>
			/// コマーシャル フレームを示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo COMR = new Id3TagInfo( "COMR", Id3TagDataType.Binary );

			/// <summary>
			/// 暗号化手法を示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo ENCR = new Id3TagInfo( "ENCR", Id3TagDataType.BinaryWithComment );

			/// <summary>
			/// イコライザー情報を示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo EQU2 = new Id3TagInfo( "EQU2", Id3TagDataType.Binary );

			/// <summary>
			/// イベント タイム コードを示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo ETCO = new Id3TagInfo( "ETCO", Id3TagDataType.Binary );

			/// <summary>
			/// パッケージ化された一般的なオブジェクトを示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo GEOB = new Id3TagInfo( "GEOB", Id3TagDataType.Binary );

			/// <summary>
			/// グループ識別子を示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo GRID = new Id3TagInfo( "GRID", Id3TagDataType.BinaryWithComment );

			/// <summary>
			/// リンク情報を示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo LINK = new Id3TagInfo( "LINK", Id3TagDataType.Link );

			/// <summary>
			/// 音楽 CD の識別子を示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo MCDI = new Id3TagInfo( "MCDI", Id3TagDataType.Binary );

			/// <summary>
			/// MPEG ロケーション ルックアップ テーブルを示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo MLLT = new Id3TagInfo( "MLLT", Id3TagDataType.Binary );

			/// <summary>
			/// 所有権フレームを示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo OWNE = new Id3TagInfo( "OWNE", Id3TagDataType.Binary );

			/// <summary>
			/// プライベートフレームを示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo PRIV = new Id3TagInfo( "PRIV", Id3TagDataType.BinaryWithComment );

			/// <summary>
			/// 再生回数を示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo PCNT = new Id3TagInfo( "PCNT", Id3TagDataType.Binary );

			/// <summary>
			/// 人気 ( 1 ～ 255 の数値で表す評価 ) を示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo POPM = new Id3TagInfo( "POPM", Id3TagDataType.BinaryWithComment );

			/// <summary>
			/// 同期位置フレームを示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo POSS = new Id3TagInfo( "POSS", Id3TagDataType.Binary );

			/// <summary>
			/// 推奨バッファサイズを示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo RBUF = new Id3TagInfo( "RBUF", Id3TagDataType.Binary );

			/// <summary>
			/// 相対ボリュームを示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo RVA2 = new Id3TagInfo( "RVA2", Id3TagDataType.Binary );

			/// <summary>
			/// リバーブ量を示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo RVRB = new Id3TagInfo( "RVRB", Id3TagDataType.Binary );

			/// <summary>
			/// シークフレームを示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo SEEK = new Id3TagInfo( "SEEK", Id3TagDataType.BinaryWithSize );

			/// <summary>
			/// 署名フレームを示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo SIGN = new Id3TagInfo( "SIGN", Id3TagDataType.Binary );

			/// <summary>
			/// タイムラインを持つ歌詞情報を示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo SYLT = new Id3TagInfo( "SYLT", Id3TagDataType.Binary );

			/// <summary>
			/// タイムラインを持つテンポ ( Bit Per Minute ) 情報を示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo SYTC = new Id3TagInfo( "SYTC", Id3TagDataType.Binary );

			/// <summary>
			/// 曲の属するアルバム、映画、劇などの名前を示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo TALB = new Id3TagInfo( "TALB", Id3TagDataType.Text );

			/// <summary>
			/// テンポ ( Bit Per Minute ) を示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo TBPM = new Id3TagInfo( "TBPM", Id3TagDataType.Text );

			/// <summary>
			/// 作曲者を示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo TCOM = new Id3TagInfo( "TCOM", Id3TagDataType.Text );

			/// <summary>
			/// ジャンルを示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo TCON = new Id3TagInfo( "TCON", Id3TagDataType.Text );

			/// <summary>
			/// 著作権情報を示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo TCOP = new Id3TagInfo( "TCOP", Id3TagDataType.Text );

			/// <summary>
			/// エンコード日時を示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo TDEN = new Id3TagInfo( "TDEN", Id3TagDataType.Text );

			/// <summary>
			/// プレイリストの遅延時間 ( 曲間の無音時間をスキップするための情報 ) を示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo TDLY = new Id3TagInfo( "TDLY", Id3TagDataType.Text );

			/// <summary>
			/// 原曲のリリース日時を示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo TDOR = new Id3TagInfo( "TDOR", Id3TagDataType.Text );

			/// <summary>
			/// 録音日時を示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo TDRC = new Id3TagInfo( "TDRC", Id3TagDataType.Text );

			/// <summary>
			/// リリース日時を示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo TDRL = new Id3TagInfo( "TDRL", Id3TagDataType.Text );

			/// <summary>
			/// タグ付け日時を示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo TDTG = new Id3TagInfo( "TDTG", Id3TagDataType.Text );

			/// <summary>
			/// ファイルのエンコードをおこなった個人または組織を示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo TENC = new Id3TagInfo( "TENC", Id3TagDataType.Text );

			/// <summary>
			/// 作詞家を示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo TEXT = new Id3TagInfo( "TEXT", Id3TagDataType.Text );

			/// <summary>
			/// オーディオファイルの形式を示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo TFLT = new Id3TagInfo( "TFLT", Id3TagDataType.Text );

			/// <summary>
			/// 関わった人々の一覧を示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo TIPL = new Id3TagInfo( "TIPL", Id3TagDataType.Text );

			/// <summary>
			/// レコーディングに要した時間 ( 形式は HHMM、計 4 桁の文字列 ) を示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo TIME = new Id3TagInfo( "TIME", Id3TagDataType.Text );

			/// <summary>
			/// グループの説明を示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo TIT1 = new Id3TagInfo( "TIT1", Id3TagDataType.Text );

			/// <summary>
			/// タイトルを示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo TIT2 = new Id3TagInfo( "TIT2", Id3TagDataType.Text );

			/// <summary>
			/// 副題やタイトルの補足説明を示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo TIT3 = new Id3TagInfo( "TIT3", Id3TagDataType.Text );

			/// <summary>
			/// 曲頭のキー ( C、Am、F♯ といった調の文字列 ) を示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo TKEY = new Id3TagInfo( "TKEY", Id3TagDataType.Text );

			/// <summary>
			/// 言語 ( ISO-639-2 として定義された 3 文字の文字列 ) を示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo TLAN = new Id3TagInfo( "TLAN", Id3TagDataType.Text );

			/// <summary>
			/// 演奏時間 ( ミリ秒単位 ) を示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo TLEN = new Id3TagInfo( "TLEN", Id3TagDataType.Text );

			/// <summary>
			/// ミュージシャン クレジット リストを示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo TMCL = new Id3TagInfo( "TMCL", Id3TagDataType.Text );

			/// <summary>
			/// メディア種別 ( 音楽 CD、レコード、...etc ) を示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo TMED = new Id3TagInfo( "TMED", Id3TagDataType.Text );

			/// <summary>
			/// 雰囲気を示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo TMOO = new Id3TagInfo( "TMOO", Id3TagDataType.Text );

			/// <summary>
			/// 原曲の属するアルバム、映画、劇などの名前を示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo TOAL = new Id3TagInfo( "TOAL", Id3TagDataType.Text );

			/// <summary>
			/// 元のファイル名を示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo TOFN = new Id3TagInfo( "TOFN", Id3TagDataType.Text );

			/// <summary>
			/// 原曲の作詞者を示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo TOLY = new Id3TagInfo( "TOLY", Id3TagDataType.Text );

			/// <summary>
			/// 原曲のアーティストを示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo TOPE = new Id3TagInfo( "TOPE", Id3TagDataType.Text );

			/// <summary>
			/// 原曲のリリース年を示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo TORY = new Id3TagInfo( "TORY", Id3TagDataType.Text );

			/// <summary>
			/// ファイルの所有者・ライセンシーを示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo TOWN = new Id3TagInfo( "TOWN", Id3TagDataType.Text );

			/// <summary>
			/// 主なアーティスト、演奏者などを示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo TPE1 = new Id3TagInfo( "TPE1", Id3TagDataType.Text );

			/// <summary>
			/// レコーディングをおこなった演奏者の追加情報を示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo TPE2 = new Id3TagInfo( "TPE2", Id3TagDataType.Text );

			/// <summary>
			/// 指揮者を示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo TPE3 = new Id3TagInfo( "TPE3", Id3TagDataType.Text );

			/// <summary>
			/// リミックスや編曲の担当者を示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo TPE4 = new Id3TagInfo( "TPE4", Id3TagDataType.Text );

			/// <summary>
			/// 曲の属するグループ内の番号 ( スラッシュ区切りの文字列 ) を示すタグ定義を取得します。
			/// 複数枚や複数の組曲で構成されるアルバムの場合、それらの総数と何番目に位置するかを表します。
			/// 例えば 3 枚組のアルバムにおいて、2 枚目に属する曲ならば "2/3" となります。
			/// </summary>
			public static readonly Id3TagInfo TPOS = new Id3TagInfo( "TPOS", Id3TagDataType.Text );

			/// <summary>
			/// 制作予告を示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo TPRO = new Id3TagInfo( "TPRO", Id3TagDataType.Text );

			/// <summary>
			/// 曲の発行元 ( レーベル ) を示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo TPUB = new Id3TagInfo( "TPUB", Id3TagDataType.Text );

			/// <summary>
			/// トラック番号 ( 書式は "番号/総数" ) を示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo TRCK = new Id3TagInfo( "TRCK", Id3TagDataType.Text );

			/// <summary>
			/// インターネットラジオ局を示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo TRSN = new Id3TagInfo( "TRSN", Id3TagDataType.Text );

			/// <summary>
			/// インターネットラジオ局の所有者を示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo TRSO = new Id3TagInfo( "TRSO", Id3TagDataType.Text );

			/// <summary>
			/// アルバムのソートオーダーを示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo TSOA = new Id3TagInfo( "TSOA", Id3TagDataType.Text );

			/// <summary>
			/// 演奏者のソートオーダーを示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo TSOP = new Id3TagInfo( "TSOP", Id3TagDataType.Text );

			/// <summary>
			/// タイトルのソートオーダーを示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo TSOT = new Id3TagInfo( "TSOT", Id3TagDataType.Text );

			/// <summary>
			/// 国際標準レコーディングコード（ International Standard Recording Code ) を示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo TSRC = new Id3TagInfo( "TSRC", Id3TagDataType.Text );

			/// <summary>
			/// エンコード設定を示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo TSSE = new Id3TagInfo( "TSSE", Id3TagDataType.Text );

			/// <summary>
			/// セットのサブタイトルを示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo TSST = new Id3TagInfo( "TSST", Id3TagDataType.Text );

			/// <summary>
			/// ユーザー定義文字情報フレームを示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo TXXX = new Id3TagInfo( "TXXX", Id3TagDataType.TextUserDefined );

			/// <summary>
			/// ファイルの一意な識別子を示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo UFID = new Id3TagInfo( "UFID", Id3TagDataType.BinaryWithComment );

			/// <summary>
			/// 使用条件を示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo USER = new Id3TagInfo( "USER", Id3TagDataType.TextWithLanguage );

			/// <summary>
			/// 歌詞を示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo USLT = new Id3TagInfo( "USLT", Id3TagDataType.TextWithCommentAndLanguage );

			/// <summary>
			/// 販売サイト URL を示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo WCOM = new Id3TagInfo( "WCOM", Id3TagDataType.Url );

			/// <summary>
			/// 著作情報サイト URL を示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo WCOP = new Id3TagInfo( "WCOP", Id3TagDataType.Url );

			/// <summary>
			/// オーディオファイルの公式サイト URL を示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo WOAF = new Id3TagInfo( "WOAF", Id3TagDataType.Url );

			/// <summary>
			/// アーティストの公式サイト URL を示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo WOAR = new Id3TagInfo( "WOAR", Id3TagDataType.Url );

			/// <summary>
			/// 音源の公式サイト URL を示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo WOAS = new Id3TagInfo( "WOAS", Id3TagDataType.Url );

			/// <summary>
			/// インターネットラジオ局の公式サイト URL を示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo WORS = new Id3TagInfo( "WORS", Id3TagDataType.Url );

			/// <summary>
			/// 支払い方法を示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo WPAY = new Id3TagInfo( "WPAY", Id3TagDataType.Url );

			/// <summary>
			/// レーベルの公式サイト URL を示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo WPUB = new Id3TagInfo( "WPUB", Id3TagDataType.Url );

			/// <summary>
			/// ユーザー定義 URL を示すタグ定義を取得します。
			/// </summary>
			public static readonly Id3TagInfo WXXX = new Id3TagInfo( "WXXX", Id3TagDataType.TextUserDefined );

			/// <summary>
			/// フレーム ID をキーとするタグ情報のディクショナリ。
			/// </summary>
			private static readonly Dictionary< string, Id3TagInfo > Infos = new Dictionary< string, Id3TagInfo >()
			{
				{ V24.ANEC.Id, V24.ANEC },
				{ V24.APIC.Id, V24.APIC },
				{ V24.COMM.Id, V24.COMM },
				{ V24.COMR.Id, V24.COMR },
				{ V24.ENCR.Id, V24.ENCR },
				{ V24.EQU2.Id, V24.EQU2 },
				{ V24.ETCO.Id, V24.ETCO },
				{ V24.GEOB.Id, V24.GEOB },
				{ V24.GRID.Id, V24.GRID },
				{ V24.LINK.Id, V24.LINK },
				{ V24.MCDI.Id, V24.MCDI },
				{ V24.MLLT.Id, V24.MLLT },
				{ V24.OWNE.Id, V24.OWNE },
				{ V24.PRIV.Id, V24.PRIV },
				{ V24.PCNT.Id, V24.PCNT },
				{ V24.POPM.Id, V24.POPM },
				{ V24.POSS.Id, V24.POSS },
				{ V24.RBUF.Id, V24.RBUF },
				{ V24.RVA2.Id, V24.RVA2 },
				{ V24.RVRB.Id, V24.RVRB },
				{ V24.SEEK.Id, V24.SEEK },
				{ V24.SIGN.Id, V24.SIGN },
				{ V24.SYLT.Id, V24.SYLT },
				{ V24.SYTC.Id, V24.SYTC },
				{ V24.TALB.Id, V24.TALB },
				{ V24.TBPM.Id, V24.TBPM },
				{ V24.TCOM.Id, V24.TCOM },
				{ V24.TCON.Id, V24.TCON },
				{ V24.TCOP.Id, V24.TCOP },
				{ V24.TDEN.Id, V24.TDEN },
				{ V24.TDLY.Id, V24.TDLY },
				{ V24.TDOR.Id, V24.TDOR },
				{ V24.TDRC.Id, V24.TDRC },
				{ V24.TDRL.Id, V24.TDRL },
				{ V24.TDTG.Id, V24.TDTG },
				{ V24.TENC.Id, V24.TENC },
				{ V24.TEXT.Id, V24.TEXT },
				{ V24.TFLT.Id, V24.TFLT },
				{ V24.TIPL.Id, V24.TIPL },
				{ V24.TIME.Id, V24.TIME },
				{ V24.TIT1.Id, V24.TIT1 },
				{ V24.TIT2.Id, V24.TIT2 },
				{ V24.TIT3.Id, V24.TIT3 },
				{ V24.TKEY.Id, V24.TKEY },
				{ V24.TLAN.Id, V24.TLAN },
				{ V24.TLEN.Id, V24.TLEN },
				{ V24.TMCL.Id, V24.TMCL },
				{ V24.TMED.Id, V24.TMED },
				{ V24.TMOO.Id, V24.TMOO },
				{ V24.TOAL.Id, V24.TOAL },
				{ V24.TOFN.Id, V24.TOFN },
				{ V24.TOLY.Id, V24.TOLY },
				{ V24.TOPE.Id, V24.TOPE },
				{ V24.TORY.Id, V24.TORY },
				{ V24.TOWN.Id, V24.TOWN },
				{ V24.TPE1.Id, V24.TPE1 },
				{ V24.TPE2.Id, V24.TPE2 },
				{ V24.TPE3.Id, V24.TPE3 },
				{ V24.TPE4.Id, V24.TPE4 },
				{ V24.TPOS.Id, V24.TPOS },
				{ V24.TPRO.Id, V24.TPRO },
				{ V24.TPUB.Id, V24.TPUB },
				{ V24.TRCK.Id, V24.TRCK },
				{ V24.TRSN.Id, V24.TRSN },
				{ V24.TRSO.Id, V24.TRSO },
				{ V24.TSOA.Id, V24.TSOA },
				{ V24.TSOP.Id, V24.TSOP },
				{ V24.TSOT.Id, V24.TSOT },
				{ V24.TSRC.Id, V24.TSRC },
				{ V24.TSSE.Id, V24.TSSE },
				{ V24.TSST.Id, V24.TSST },
				{ V24.TXXX.Id, V24.TXXX },
				{ V24.UFID.Id, V24.UFID },
				{ V24.USER.Id, V24.USER },
				{ V24.USLT.Id, V24.USLT },
				{ V24.WCOM.Id, V24.WCOM },
				{ V24.WCOP.Id, V24.WCOP },
				{ V24.WOAF.Id, V24.WOAF },
				{ V24.WOAR.Id, V24.WOAR },
				{ V24.WOAS.Id, V24.WOAS },
				{ V24.WORS.Id, V24.WORS },
				{ V24.WPAY.Id, V24.WPAY },
				{ V24.WPUB.Id, V24.WPUB },
				{ V24.WXXX.Id, V24.WXXX }
			};
		}
	}
}
