﻿using System;
using System.IO;
using System.Text;
using ComponentAce.Compression.Libs.zlib;
using System.Diagnostics;

namespace Owl.Id3
{
	/// <summary>
	/// ID3v1、ID3v2.2 ～ 4 のタグ情報をダンプします。
	/// </summary>
	class TagDumper : IDisposable
	{
		/// <summary>
		/// インスタンスを初期化します。
		/// </summary>
		/// <param name="filePath">ファイルのパス情報。</param>
		/// <param name="dumpFilePath">ダンプ先となるテキストファイルへのパス情報。</param>
		public TagDumper( string filePath, string dumpFilePath )
		{
			this._reader   = new FileStream( filePath, FileMode.Open, FileAccess.Read, FileShare.Read );
			this._writer   = new StreamWriter( dumpFilePath, true, Encoding.Unicode );
			this._filePath = filePath;
		}
	
		/// <summary>
		/// 文字コードを指定してバイト配列を文字列に変換します。
		/// </summary>
		/// <param name="encode">文字コード。</param>
		/// <param name="value">バイト配列。</param>
		/// <param name="index">バイト配列の変換を開始する位置。</param>
		/// <param name="count">変換をおこなうバイト数。</param>
		/// <returns>文字列。</returns>
		private string ConvertEncodeString( Id3TextEncoding encode, byte[] value, int index, int count )
		{
			var terminator = this.FindStringTerminate( value, index, count, ( encode == Id3TextEncoding.Utf16 || encode == Id3TextEncoding.Utf16BE ) );
			if( terminator <= 0 ) { terminator = count; }

			// 終端文字の直前まで読み取る
			terminator -= index;
			switch( encode )
			{
			case Id3TextEncoding.Utf16:   return StringUtility.GetStringUtf16WithBom( value, index, terminator );
			case Id3TextEncoding.Utf16BE: return Encoding.BigEndianUnicode.GetString( value, index, terminator );
			case Id3TextEncoding.Utf8:    return Encoding.UTF8.GetString( value, index, terminator );
			default:                      return Encoding.Default.GetString( value, index, terminator );
			}
		}

		/// <summary>
		/// リソースを解放します。
		/// </summary>
		public void Dispose()
		{
			if( this._reader != null )
			{
				this._reader.Dispose();
				this._reader = null;
			}

			if( this._writer != null )
			{
				this._writer.Dispose();
				this._writer = null;
			}

			GC.SuppressFinalize( this );
		}

		/// <summary>
		/// フレームをダンプします。
		/// </summary>
		/// <param name="reader">タグ情報を読み取るためのストリーム。</param>
		/// <param name="writer">タグ情報を出力するためのストリーム。</param>
		/// <param name="version">バージョン情報。</param>
		/// <param name="tagSize">タグ情報のサイズ。</param>
		private void DumpFramesV2( Stream reader, StreamWriter writer, byte[] version, uint tagSize )
		{
			writer.WriteLine( "\nFrames\nType\tValue" );

			Func< Stream, StreamWriter, uint, int, uint > dumpFrame = null;
			if( version[ 0 ] == 2 )
			{
				dumpFrame = new Func< Stream, StreamWriter, uint, int, uint >( DumpFrameV22 );
			}
			else if( version[ 0 ] == 3 && version[ 1 ] == 0 )
			{
				dumpFrame = new Func< Stream, StreamWriter, uint, int, uint >( DumpFrameV23 );
			}
			else
			{
				dumpFrame = new Func< Stream, StreamWriter, uint, int, uint >( DumpFrameV24 );
			}

			var remainSize = tagSize;
			var index      = 0;
			while( remainSize > 0 )
			{
				try
				{
					remainSize -= dumpFrame( reader, writer, remainSize, index++ );
				}
				catch( Exception e )
				{
					Debug.WriteLine( e );
					break;
				}
			}
		}

		/// <summary>
		/// フレームをダンプします。
		/// </summary>
		/// <param name="reader">タグ情報を読み取るためのストリーム。</param>
		/// <param name="writer">タグ情報を出力するためのストリーム。</param>
		/// <param name="info">タグ情報。</param>
		/// <param name="size">タグ情報のサイズ。</param>
		/// <param name="index">フレームの通し番号。</param>
		/// <param name="flags">フレームのフラグ。</param>
		private void DumpFrameValue( Stream reader, StreamWriter writer, Id3TagInfo info, uint size, int index, FrameFlags flags )
		{
			// 圧縮されているなら展開する
			Stream data = reader;
			if( ( flags & FrameFlags.Compressed ) == FrameFlags.Compressed )
			{
				var m = new MemoryStream();
				this.UnZip( reader, m );
				data = m;
			}

			switch( info.Type )
			{
			case Id3TagDataType.Text:                       this.DumpFrameValueText( data, writer, size, index );                       break;
			case Id3TagDataType.TextUserDefined:            this.DumpFrameValueTextUserDefined( data, writer, size, index );            break;
			case Id3TagDataType.TextWithCommentAndLanguage: this.DumpFrameValueTextWithCommentAndLanguage( data, writer, size, index ); break;
			case Id3TagDataType.Url:                        this.DumpFrameValueUrl( data, writer, size, index );                        break;

			// 未対応のデータはとりあえずバイナリにしておく
			default:
				writer.WriteLine( "[ {0} ] Value\t byte[ {1} ]", index, size );
				reader.Seek( size, SeekOrigin.Current );
				break;
			}
		}

		/// <summary>
		/// テキスト情報フレームをダンプします。
		/// </summary>
		/// <param name="reader">タグ情報を読み取るためのストリーム。</param>
		/// <param name="writer">タグ情報を出力するためのストリーム。</param>
		/// <param name="size">タグ情報のサイズ。</param>
		/// <param name="index">フレームの通し番号。</param>
		private void DumpFrameValueText( Stream reader, StreamWriter writer, uint size, int index )
		{
			// エンコード指定は 1 バイトなので、文字列を読み取るサイズ指定は必ず 1 バイト分、差し引く必要がある
			var encode = ( Id3TextEncoding )reader.ReadByte();
			var b      = reader.ReadBytes( ( int )( size - 1 ) );
			writer.WriteLine( "[ {0} ] Value\t\"{1}\", \"{2}\"", index, encode, this.ConvertEncodeString( encode, b, 0, b.Length ) );
		}

		/// <summary>
		/// ユーザー定義テキスト情報フレームをダンプします。
		/// </summary>
		/// <param name="reader">タグ情報を読み取るためのストリーム。</param>
		/// <param name="writer">タグ情報を出力するためのストリーム。</param>
		/// <param name="size">タグ情報のサイズ。</param>
		/// <param name="index">フレームの通し番号。</param>
		private void DumpFrameValueTextUserDefined( Stream reader, StreamWriter writer, uint size, int index )
		{
			var encode = ( Id3TextEncoding )reader.ReadByte();
			writer.WriteLine( "[ {0} ] Value\t\"{1}\", {2}", index, encode, this.GetTextWithDescription( reader.ReadBytes( ( int )( size - 1 ) ), encode ) );
		}

		/// <summary>
		/// 言語・説明付きテキスト情報フレームをダンプします。
		/// </summary>
		/// <param name="reader">タグ情報を読み取るためのストリーム。</param>
		/// <param name="writer">タグ情報を出力するためのストリーム。</param>
		/// <param name="size">タグ情報のサイズ。</param>
		/// <param name="index">フレームの通し番号。</param>
		private void DumpFrameValueTextWithCommentAndLanguage( Stream reader, StreamWriter writer, uint size, int index )
		{
			var encode = ( Id3TextEncoding )reader.ReadByte();
			var lang   = reader.ReadStringAscii( 3 );
			writer.WriteLine( "[ {0} ] Value\t\"{1}\", \"{2}\", {3}", index, encode, lang, this.GetTextWithDescription( reader.ReadBytes( ( int )( size - 4 ) ), encode ) );
		}

		/// <summary>
		/// URL フレームをダンプします。
		/// </summary>
		/// <param name="reader">タグ情報を読み取るためのストリーム。</param>
		/// <param name="writer">タグ情報を出力するためのストリーム。</param>
		/// <param name="size">タグ情報のサイズ。</param>
		/// <param name="index">フレームの通し番号。</param>
		private void DumpFrameValueUrl( Stream reader, StreamWriter writer, uint size, int index )
		{
			var str = reader.ReadStringDefault( ( int )size );
			writer.WriteLine( "[ {0} ] Value\t{1}", index, str );
		}

		/// <summary>
		/// ID3v2.2 のフレームをダンプします。
		/// </summary>
		/// <param name="reader">タグ情報を読み取るためのストリーム。</param>
		/// <param name="writer">タグ情報を出力するためのストリーム。</param>
		/// <param name="remainSize">タグ情報全体の残りサイズ。</param>
		/// <param name="index">フレームの通し番号。</param>
		/// <returns>読み取ったデータ サイズ。</returns>
		/// <exception cref="UnsupportedFormatException">未サポートのデータ形式です。</exception>
		private uint DumpFrameV22( Stream reader, StreamWriter writer, uint remainSize, int index )
		{
			// フレームヘッダがないので何もしない
			if( remainSize < 6 ) { return 0; }

			// 未定義フレームなら例外
			var id = reader.ReadStringAscii( 3 );
			var info = Id3DefinedTags.V22.GetInfo( id );
			if( info == null ) { throw new UnsupportedFormatException( String.Format( "Undefined frame \"{0}\".", id ) ); }
			writer.WriteLine( "[ {0} ] ID\t{1}", index, id );

			uint size = 0;
			{
				var b = reader.ReadBytes( 3 );
				size = ( ( uint )( b[ 0 ] << 16 ) | ( uint )( b[ 1 ] << 8 ) | ( uint )( b[ 2 ] ) ); 
				writer.WriteLine( "[ {0} ] Size\t{1}", index, size );
			}


			this.DumpFrameValue( reader, writer, info, size, index, FrameFlags.None );
			return size + 6;
		}

		/// <summary>
		/// ID3v2.3 のフレームをダンプします。
		/// </summary>
		/// <param name="reader">タグ情報を読み取るためのストリーム。</param>
		/// <param name="writer">タグ情報を出力するためのストリーム。</param>
		/// <param name="remainSize">タグ情報全体の残りサイズ。</param>
		/// <param name="index">フレームの通し番号。</param>
		/// <returns>読み取ったデータ サイズ。</returns>
		/// <exception cref="UnsupportedFormatException">未サポートのデータ形式です。</exception>
		private uint DumpFrameV23( Stream reader, StreamWriter writer, uint remainSize, int index )
		{
			// フレームヘッダがないので何もしない
			if( remainSize < 10 ) { return 0; }

			// 未定義フレームなら例外
			var id = reader.ReadStringAscii( 4 );
			var info = Id3DefinedTags.V22.GetInfo( id );
			if( info == null ) { throw new UnsupportedFormatException( String.Format( "Undefined frame \"{0}\".", id ) ); }
			writer.WriteLine( "[ {0} ] ID\t{1}", index, id );

			uint size = 0;
			{
				var b = reader.ReadBytes( 4 );
				size = ( ( uint )( b[ 0 ] << 24 ) | ( uint )( b[ 1 ] << 16 ) | ( uint )( b[ 2 ] << 8 ) | ( uint )( b[ 3 ] ) );
				writer.WriteLine( "[ {0} ] Size\t{1}", index, size );
			}


			// フラグ
			uint optionalSize = 0;
			FrameFlags flags;
			{
				var b = reader.ReadBytes( 2 );
				flags = this.GetFrameFlags( b );
				writer.WriteLine( "[ {0} ] Status, Format flags\t{1}, {2}", index, Convert.ToString( b[ 0 ], 2 ), Convert.ToString( b[ 1 ], 2 ) );

				// 圧縮 ( 伸長後のサイズ = 4 )、暗号化 ( フラグ = 1 )、グループ化 ( 識別子 = 1 ) をスキップ
				if( ( flags & FrameFlags.Compressed ) == FrameFlags.Compressed ) {  optionalSize += 4; }
				if( ( flags & FrameFlags.Encrypted  ) == FrameFlags.Encrypted  ) {  optionalSize += 1; }
				if( ( flags & FrameFlags.Grouping   ) == FrameFlags.Grouping   ) {  optionalSize += 1; }
			}

			this.DumpFrameValue( reader, writer, info, size, index, flags );
			return size + 10 + optionalSize;
		}

		/// <summary>
		/// ID3v2.4 のフレームをダンプします。
		/// </summary>
		/// <param name="reader">タグ情報を読み取るためのストリーム。</param>
		/// <param name="writer">タグ情報を出力するためのストリーム。</param>
		/// <param name="remainSize">タグ情報全体の残りサイズ。</param>
		/// <param name="index">フレームの通し番号。</param>
		/// <returns>読み取ったデータ サイズ。</returns>
		/// <exception cref="UnsupportedFormatException">未サポートのデータ形式です。</exception>
		private uint DumpFrameV24( Stream reader, StreamWriter writer, uint remainSize, int index )
		{
			// フレームヘッダがないので何もしない
			if( remainSize < 10 ) { return 0; }

			// 未定義フレームなら例外
			var id = reader.ReadStringAscii( 4 );
			var info = Id3DefinedTags.V22.GetInfo( id );
			if( info == null ) { throw new UnsupportedFormatException( String.Format( "Undefined frame \"{0}\".", id ) ); }		
			writer.WriteLine( "[ {0} ] ID\t{1}", index, id );

			uint size = 0;
			{
				var b = reader.ReadBytes( 4 );
				size = ( ( uint )( b[ 0 ] << 24 ) | ( uint )( b[ 1 ] << 16 ) | ( uint )( b[ 2 ] << 8 ) | ( uint )( b[ 3 ] ) );
				writer.WriteLine( "[ {0} ] Size\t{1}", index, size );
			}

			// フラグ
			uint optionalSize = 0;
			FrameFlags flags;
			{
				var b = reader.ReadBytes( 2 );
				flags = this.GetFrameFlags( b );
				writer.WriteLine( "[ {0} ] Status, Format flags\t{1}, {2}", index, Convert.ToString( b[ 0 ], 2 ), Convert.ToString( b[ 1 ], 2 ) );

				// 圧縮 ( 伸長後のサイズ = 4 )、暗号化 ( フラグ = 1 )、グループ化 ( 識別子 = 1 ) をスキップ
				if( ( flags & FrameFlags.Compressed ) == FrameFlags.Compressed ) {  optionalSize += 4; }
				if( ( flags & FrameFlags.Encrypted  ) == FrameFlags.Encrypted  ) {  optionalSize += 1; }
				if( ( flags & FrameFlags.Grouping   ) == FrameFlags.Grouping   ) {  optionalSize += 1; }
			}

			this.DumpFrameValue( reader, writer, info, size, index, flags );
			return size + 10 + optionalSize;
		}
		
		/// <summary>
		/// 拡張ヘッダをダンプし、新しいタグ情報のサイズを算出します。
		/// </summary>
		/// <param name="reader">タグ情報を読み取るためのストリーム。</param>
		/// <param name="writer">タグ情報を出力するためのストリーム。</param>
		/// <param name="version">バージョン情報。</param>
		/// <param name="headerFlags">ヘッダのフラグ。</param>
		/// <param name="tagSize">タグ情報のサイズ。</param>
		/// <returns>新しいタグ情報のサイズ。</returns>
		private uint DumpExtenedeHeaderV2( Stream reader, StreamWriter writer, byte[] version, byte headerFlags, uint tagSize )
		{
			// 2.3 以降のみ
			if( version[ 0 ] < 3 || ( ( headerFlags & 0x40 ) == 0 ) ) { return tagSize; }

			writer.WriteLine( "\nExtended Header\nType\tValue" );

			// 拡張ヘッダのサイズとデータ部分を差し引く
			var exSize = this.DumpSizeV2( reader, writer, "Extended header size" );
			tagSize -= exSize + 4;

			// サイズ以降のデータ構成はバージョンによって異なる
			if( version[ 0 ] == 3 && version[ 1 ] == 0 )
			{
				var exFlags = reader.ReadBytes( 2 );
				writer.WriteLine( "Flags\t{0:X2}, {1:X2}", exFlags[ 0 ], exFlags[ 1 ] );
				this.DumpSizeV2( reader, writer, "Padding size" );

				if( ( exFlags[ 0 ] & 0x80 ) != 0 )
				{
					var crc = reader.ReadBytes( 4 );
					writer.WriteLine( "CRC\t{0:X2}, {1:X2}, {2:X2}, {3:X2}", crc[ 0 ], crc[ 1 ], crc[ 2 ], crc[ 3 ] );
				}
			}
			else
			{
				var flagLength = reader.ReadBytes( 1 );
				writer.WriteLine( "Flag length\t{0}", flagLength[ 0 ] );

				var exFlags        = reader.ReadBytes( 1 );
				var hasCrc         = ( ( exFlags[ 0 ] & 0x20 ) != 0 );
				var isRestrictions = ( ( exFlags[ 0 ] & 0x10 ) != 0 );
				writer.WriteLine( "Flags\t{0:X2} : Tag updated = {1}, CRC = {2}, Tag restrictions = {3}", exFlags[ 0 ], ( ( exFlags[ 0 ] & 0x40 ) != 0 ), hasCrc, isRestrictions );

				var remainSize = exSize - 4;

				if( hasCrc && remainSize >= 5 )
				{
					var crc = reader.ReadBytes( 5 );
					writer.WriteLine( "CRC\t{0:X2}, {1:X2}, {2:X2}, {3:X2}, {4:X2}", crc[ 0 ], crc[ 1 ], crc[ 2 ], crc[ 3 ], crc[ 4 ] );
					remainSize -= 5;
				}

				if( isRestrictions && remainSize >= 1 )
				{
					var b = reader.ReadBytes( 1 );
					writer.WriteLine( "Tag restrictions\t{0}", Convert.ToString( b[ 0 ], 2 ) );
					--remainSize;
				}
			}

			return tagSize;
		}

		/// <summary>
		/// ヘッダのフラグをダンプします。
		/// </summary>
		/// <param name="reader">タグ情報を読み取るためのストリーム。</param>
		/// <param name="writer">タグ情報を出力するためのストリーム。</param>
		/// <returns>フラグ。</returns>
		private byte DumpHeaderFlagsV2( Stream reader, StreamWriter writer )
		{
			// 8 ビットの値について「非同期化」、「拡張ヘッダ所有」、「実験タグ」の順にビットフラグが設けられている
			var b = reader.ReadBytes( 1 );
			writer.WriteLine( "Flags\t{0} : Unsync = {1}, Extended Header = {2}, Experimental = {3}", b[ 0 ], ( ( b[ 0 ] & 0x80 ) != 0 ), ( ( b[ 0 ] & 0x40 ) != 0 ), ( ( b[ 0 ] & 0x20 ) != 0 ) );

			return b[ 0 ];
		}

		/// <summary>
		/// ID3v2 のタグ情報サイズを取得します。
		/// サイズは合計 4 バイトのデータで、第 7 ビットが 0 になるように設定されいるので実質的なデータは 3 バイト ( 28 ビット ) となります。
		/// 例えば 257 バイトの場合は { 00, 00, 02, 01 } のようにレイアウトされます。
		/// サイズにはヘッダ部分を含みません。
		/// </summary>
		/// <param name="reader">タグ情報を読み取るためのストリーム。</param>
		/// <param name="writer">タグ情報を出力するためのストリーム。</param>
		/// <param name="type">サイズの種類。</param>
		/// <returns>サイズ。</returns>
		private uint DumpSizeV2( Stream reader, StreamWriter writer, string type )
		{
			var b    = reader.ReadBytes( 4 );
			var size = ( ( uint )( ( b[ 0 ] & 0x7f ) << 21 ) | ( uint )( ( b[ 1 ] & 0x7f ) << 14 ) | ( uint )( ( b[ 2 ] & 0x7f ) << 7 ) | ( uint )( b[ 0 ] & 0x7f ) );

			writer.WriteLine( "{0}\t{1}", type, size );
			return size;
		}

		/// <summary>
		/// バージョン情報をダンプします。
		/// </summary>
		/// <param name="reader">タグ情報を読み取るためのストリーム。</param>
		/// <param name="writer">タグ情報を出力するためのストリーム。</param>
		/// <returns>バージョン情報。</returns>
		/// <exception cref="NotSupportedException">サポート外のファイル形式です。</exception>
		private byte[] DumpVersionV2( Stream reader, StreamWriter writer )
		{
			// v2.2.0、v2.3.0、v2.3.1 ( v2.4 )、v2.4.0 のみサポート対象とする
			var b = reader.ReadBytes( 2 );
			if( !( ( b[ 0 ] == 2 && b[ 1 ] == 0 ) || ( b[ 0 ] == 3 && ( b[ 1 ] == 0 ) || b[ 1 ] == 1 ) || ( b[ 0 ] == 4 && b[ 1 ] == 0 ) ) )
			{
				throw new NotSupportedException( String.Format( "Unsupported version \"{0}.{1}\".", b[ 0 ], b[ 1 ] ) );
			}

			writer.WriteLine( "Version\tv2.{0}.{1}", b[ 0 ], b[ 1 ] );
			return b;
		}

		/// <summary>
		/// ID3v1 のタグ情報をダンプします。
		/// </summary>
		/// <exception cref="NotSupportedException">サポート外のファイル形式です。</exception>
		public void DumpV1()
		{
			var reader = this._reader;
			var writer = this._writer;

			// ID3v1 チェック
			reader.Seek( -128, SeekOrigin.End );
			if( !this.HasTagV1( reader ) ) { throw new NotSupportedException( "Id3v1 tag was not found." ); }
			
			writer.WriteLine( "\nID3v1\nType\tValue" );

			writer.WriteLine( "Title\t{0}",  reader.ReadStringDefault( 30 ) );
			writer.WriteLine( "Artist\t{0}", reader.ReadStringDefault( 30 ) );
			writer.WriteLine( "Album\t{0}",  reader.ReadStringDefault( 30 ) );
			writer.WriteLine( "Year\t{0}",   reader.ReadStringAscii( 4 )    );

			// コメントとトラック番号
			{
				int		track = 0;
				string	comment;

				var b = reader.ReadBytes( 30 );
				if( b[ 28 ] == 0 )
				{
					// 余白はゼロで埋められるため、29 バイト目がゼロならば以降はゼロまたはトラック番号と考えられる。
					// 仮にトラック番号が未指定でも、ゼロとして取得される。
					// 通常、トラック番号は 1 から始まるので、ゼロであれば無効値としてもよい。
					//
					track = b[ 29 ];
					comment = StringUtility.TrimLastNull( Encoding.Default.GetString( b, 0, 28 ) );
				}
				else
				{
					comment = StringUtility.TrimLastNull( Encoding.Default.GetString( b ) );
				}

				writer.WriteLine( "Comment\t{0}",      comment );
				writer.WriteLine( "Track Number\t{0}", track   );
			}

			writer.WriteLine( "Genre\t{0}", reader.ReadByte() );
		}

		/// <summary>
		/// ID3v2 のタグ情報をダンプします。
		/// </summary>
		/// <exception cref="NotSupportedException">サポート外のファイル形式です。</exception>
		public void DumpV2()
		{
			var reader = this._reader;
			var writer = this._writer;

			// ID3v2 タグ ( 3 )
			reader.Seek( 0, SeekOrigin.Begin );
			if( !this.HasTagV2( reader ) ) { throw new NotSupportedException( "Id3v2 tag was not found." ); }
			writer.WriteLine( "\nID3v2\n\nHeader\nType\tValue" );

			// バージョン ( 2 )
			byte[] version = this.DumpVersionV2( reader, writer );

			// フラグ ( 1 )
			var flags = this.DumpHeaderFlagsV2( reader, writer );

			// サイズ ( 4 )
			var size = this.DumpSizeV2( reader, writer, "Tag size" );

			// 非同期化されているならば、解除後のデータを生成して読み取りストリームを変更
			if( ( flags & 0x80 ) != 0 )
			{
				var m  = new MemoryStream();
				this.Sync( reader, m, size );
				reader = m;
				size   = ( uint )reader.Length;
			}

			size = this.DumpExtenedeHeaderV2( reader, writer, version, flags, size );

			// フレーム出力
			this.DumpFramesV2( reader, writer, version, size );
		}

		/// <summary>
		/// 指定されたバイト配列から文字列終端 ( ワイド文字なら $00 00、それ以外は $00 ) を探します。
		/// </summary>
		/// <param name="value">バイト配列。</param>
		/// <param name="index">検索を開始する位置。</param>
		/// <param name="count">検索の終了位置。</param>
		/// <param name="isWideChar">バイト配列がワイド文字なら true。それ以外は false。</param>
		/// <returns>成功時は終端位置。それ以外は -1。</returns>
		private int FindStringTerminate( byte[] value, int index, int count, bool isWideChar )
		{
			var max = ( index + count < value.Length ? count : value.Length );
			for( var i = index; i < max; ++i )
			{
				if( value[ i ] == 0x00 && ( !isWideChar || ( i + 1 < value.Length && value[ i + 1 ] == 0x00 ) ) )
				{
					return i;
				}
			}

			return -1;
		}

		/// <summary>
		/// 説明付きテキストの内容を文字列として取得します。
		/// </summary>
		/// <param name="value">説明付きテキストの内容となるバイト配列。</param>
		/// <param name="encode">文字コード。</param>
		/// <returns>文字列。</returns>
		private string GetTextWithDescription( byte[] value, Id3TextEncoding encode )
		{
			var isWideChar = ( encode == Id3TextEncoding.Utf16 || encode == Id3TextEncoding.Utf16BE );
			
			// 区切り位置の始点を得る
			var separator  = this.FindStringTerminate( value, 0, value.Length, isWideChar );
			if( separator == -1 ) { return "\"\", \"\""; }

			// テキストの読み出し位置 ( 区切り位置 + 区切りの長さ ) を算出
			var offset = ( isWideChar ? separator + 2 : separator + 1 );

			var desc = ( separator == 0         ? String.Empty : this.ConvertEncodeString( encode, value, 0, separator - 1              ) );
			var text = ( offset >= value.Length ? String.Empty : this.ConvertEncodeString( encode, value, offset, value.Length - offset ) );

			return String.Format( "\"{0}\", \"{1}\"", desc, text );
		}

		/// <summary>
		/// バイト配列からフレームのフラグを取得します。
		/// </summary>
		/// <param name="value">バイト配列。</param>
		/// <returns>フラグ。</returns>
		private FrameFlags GetFrameFlags( byte[] value )
		{
			FrameFlags flags = FrameFlags.None;
			if( ( value[ 0 ] & 0x80 ) != 0 ) { flags |= FrameFlags.RemoveFrameOnUpdateTag;  }
			if( ( value[ 0 ] & 0x40 ) != 0 ) { flags |= FrameFlags.RemoveFrameOnUpdateFile; }
			if( ( value[ 0 ] & 0x20 ) != 0 ) { flags |= FrameFlags.Readonly;                }
			if( ( value[ 1 ] & 0x80 ) != 0 ) { flags |= FrameFlags.Compressed;              }
			if( ( value[ 1 ] & 0x40 ) != 0 ) { flags |= FrameFlags.Encrypted;               }
			if( ( value[ 1 ] & 0x20 ) != 0 ) { flags |= FrameFlags.Grouping;                }

			return flags;
		}

		/// <summary>
		/// ID3v1 のタグ情報を持つことを調べます。
		/// </summary>
		/// <param name="reader">タグ情報を読み取るためのストリーム。</param>
		/// <returns>タグ情報を持つ場合は true。それ以外は false。</returns>
		private bool HasTagV1( Stream reader )
		{
			var b = reader.ReadBytes( 3 );
			return ( b[ 0 ] == TagDumper.Id3v1Tag[ 0 ] && b[ 1 ] == TagDumper.Id3v1Tag[ 1 ] && b[ 2 ] == TagDumper.Id3v1Tag[ 2 ] );
		}

		/// <summary>
		/// ID3v2 のタグ情報を持つことを調べます。
		/// </summary>
		/// <param name="reader">タグ情報を読み取るためのストリーム。</param>
		/// <returns>タグ情報を持つ場合は true。それ以外は false。</returns>
		private bool HasTagV2( Stream reader )
		{
			var b = reader.ReadBytes( 3 );
			return ( b[ 0 ] == TagDumper.Id3v2Tag[ 0 ] && b[ 1 ] == TagDumper.Id3v2Tag[ 1 ] && b[ 2 ] == TagDumper.Id3v2Tag[ 2 ] );
		}

		/// <summary>
		/// 非同期化されたデータを復元しますします。
		/// </summary>
		/// <param name="src">非同期化を解除すべきデータを持つストリーム。</param>
		/// <param name="dest">非同期化を解除したデータを書き込むストリーム。</param>
		/// <param name="size">非同期化された状態のデータ サイズ。</param>
		private void Sync( Stream src, Stream dest, uint size )
		{
			byte last = 0;
			for( var i = 0; i < size; ++i )
			{
				var b = src.ReadBytes( 1 )[ 0 ];
				if( !( last == 0xFF && b == 0x00 ) )
				{
					dest.WriteByte( b );
				}

				last = b;
			}
		}

		/// <summary>
		/// 圧縮されたデータを復元します。
		/// </summary>
		/// <param name="src">圧縮されたデータへのストリーム。</param>
		/// <param name="dest">復元されたデータを書き込むストリーム</param>
		private void UnZip( Stream src, Stream dest )
		{
			var reader = new ZInputStream( src );
			var buffer = new byte[ 1024 ];

			while( true )
			{
				var length = reader.read( buffer, 0, buffer.Length );
				if( length <= 0 ) { break; }

				dest.Write( buffer, 0, length );
			}
		}

		/// <summary>
		/// ファイルのパス情報。
		/// </summary>
		private string _filePath;

		/// <summary>
		/// タグ情報を読み取るためのストリーム。
		/// </summary>
		private Stream _reader;

		/// <summary>
		/// タグ情報を出力するためのストリーム。
		/// </summary>
		private StreamWriter _writer;

		/// <summary>
		/// I*D3v1 のタグ始点をあらわすバイト配列。ASCII 文字列で表すと "TAG" となる。
		/// </summary>
		private static readonly byte[] Id3v1Tag = { 0x54, 0x41, 0x47 };

		/// <summary>
		/// ID3v2 のタグ始点をあらわすバイト配列。ASCII 文字列で表すと "ID3" となる。
		/// </summary>
		private static readonly byte[] Id3v2Tag = { 0x49, 0x44, 0x33 };
	}
}
